/**
 * \addtogroup osSWinc
 * @{
 * \addtogroup fatfs
 * @{
 */
/** \file ffconf.h
 *
 *  (Grande) fatFS configuration options
 *
 *  This is the include file for ChaN's (grande) fatFS for small memory
 *  cards (SMC, MMC) and  FAT file system functions. It contains basic
 *  option settings.
 *
 *  Copyright (c) 2012, ChaN, all right reserved. <br />
 *
 *  Modification for \ref intro_sec "weAutSys" by A. Weinert <br />
 \ref copyLicNote " Copyright &copy; " 2012 Albrecht Weinert <br />
 *  weinert - automation, Bochum <br />
 *  weinert-automation.de  weAut.de  a-weinert.de
 *
 *  Look at file \ref ff.h "fatFS/ff.h" for details.
 *
 * 2
 * 25.01.2017
 *
 */
/** @} */
/** @} */ // osSWinc

#ifndef _FFCONF
#define _FFCONF 4004  /* Revision ID */
#define _weAutSys 1

//--- Functions and buffer configurations  ---------------------------------

/**
 * \addtogroup osSWinc
 * @{
 * \addtogroup fatfs
 * @{
 */


/** Only read functions
 *
 *  In a read only configuration all write functions would be removed:
 *  \ref f_write, \ref f_sync, \ref f_unlink, \ref f_mkdir, \ref f_chmod,
 *  \ref f_rename, \ref f_truncate as well as \ref f_getfree (then useless).
 *
 *  Values:  0 = read/write or 1 = read only
 *  \showinitializer
 */
#define _FS_READONLY  0

/** Omit functions to save (program / flash) memory
 *
 *  The _FS_MINIMIZE option defines minimisation level to remove some
 *  functions. <br />
 *  0: No reduction, except on \ref intro_sec "weAutSys" omission
 *     of f_getfree and f_lseek *a) <br />
 *  1: Omit  f_stat, f_getfree, f_unlink, f_mkdir, f_chmod, f_truncate
 *     and f_rename. <br />
 *  2: Additionally omit f_opendir and f_readdir. <br />
 *  3: Omit  f_lseek also.
 *
 *  Note *a): f_getfree and f_lseek follow sectors until certain conditions
 *  are met. This leads to unpredictable run times &mdash; often long enough
 *  to fire a watchdog reset.
 *
 *  \showinitializer
 */
#define _FS_MINIMIZE 0

#define _USE_STRFUNC 0 //!< 0: disable or 1-2: enable string functions \showinitializer

#define _USE_FASTSEEK 0 //!< 0: disable or 1: enable the fast seek feature \showinitializer


//----- Locale and namespace configurations

/** The code page used
 *
 *  The _CODE_PAGE specifies the OEM code page to be used on the target
 *  system. Incorrect setting of the code page can cause a file open failure.
 *
 *   932 - Japanese Shift-JIS (DBCS, OEM, Windows) <br />
 *   936 - Simplified Chinese GBK (DBCS, OEM, Windows) <br />
 *   949 - Korean (DBCS, OEM, Windows) <br />
 *   950 - Traditional Chinese Big5 (DBCS, OEM, Windows) <br />
 *  1250 - Central Europe (Windows) <br />
 *  1251 - Cyrillic (Windows) <br />
 *  1252 - Latin 1 (Windows) <br />
 *  1253 - Greek (Windows) <br />
 *  1254 - Turkish (Windows) <br />
 *  1255 - Hebrew (Windows) <br />
 *  1256 - Arabic (Windows) <br />
 *  1257 - Baltic (Windows) <br />
 *  1258 - Vietnam (OEM, Windows) <br />
 *   437 - U.S. (OEM) <br />
 *   720 - Arabic (OEM) <br />
 *   737 - Greek (OEM) <br />
 *   775 - Baltic (OEM) <br />
 *   850 - Multilingual Latin 1 (OEM) <br />
 *   858 - Multilingual Latin 1 + Euro (OEM) <br />
 *   852 - Latin 2 (OEM) <br />
 *   855 - Cyrillic (OEM) <br />
 *   866 - Russian (OEM) <br />
 *   857 - Turkish (OEM) <br />
 *   862 - Hebrew (OEM) <br />
 *   874 - Thai (OEM, Windows) <br />
 *    1  - ASCII only (Valid for non \ref _USE_LFN "LFN" configuration)
 *
 * \showinitializer
 */
#define _CODE_PAGE 1

/** The long file name feature
 *
 *  0: Disable LFN feature. _MAX_LFN and _LFN_UNICODE have no effect. <br />
 *  1: Enable LFN with static working buffer on the BSS.
 *     Always NOT reentrant. <br />
 *  2: Enable LFN with dynamic working buffer on the stack. <br />
 *  3: Enable LFN with dynamic working buffer on the heap. <br />
 *
 *  The LFN working buffer occupies (_MAX_LFN + 1) * 2 bytes. To enable LFN,
 *  the Unicode handling functions ff_convert() and ff_wtoupper() must be
 *  added to the project. To use heap, the memory control functions
 *  ff_memalloc() and ff_memfree() must be added to the project.
 *
 *  For small systems and SMCs this feature is switched off for space,
 *  performance and license reasons. (Microsoft holds patents for long
 *  file names on FAT file systems.) One extra drawback of using only short
 *  file names is their using only upper case letters.
 *
 *  \showinitializer
 */
#define  _USE_LFN 0
#define  _MAX_LFN 255 //!< Maximum LFN length to handle (12 to 255) \showinitializer
#define  _LFN_UNICODE 0 //!< 0: ANSI/OEM or 1: Unicode \showinitializer

/** The relative path feature
 *
 *  0: Disable relative path feature and remove related functions. <br />
 *  1: Enable relative path functions f_chdrive() and f_chdir(). <br />
 *  2: f_getcwd() is available, also.
 *
 *  Note that output of \ref f_readdir() is affected by this option. <br />
 *  For small systems and SMCs this feature is switched off.
 *
 *  \showinitializer
 */
#define _FS_RPATH 0


//---   Physical Drive Configurations

#define _VOLUMES 1 //!< Number of volumes (logical drives) to be used \showinitializer


/** Allow multiple partitions
 *
 *  legal values: <br />
 *  &nbsp; 0: single partition, <br />
 *  &nbsp; 1/2: enable multiple partition
 *
 *  When set to 0, each volume is bound to the same physical drive number and
 *  can mount only the first primary partition. When it is set to 1 or 2, each
 *  volume is tied to the partitions listed in (then defined) VolToPart[].
 *  When set to 2 a f_fisk() function is defined additionally.
 *
 *  Hint: For small memory card (SMC, MMC) implementation on small
 *        controllers _MULTI_PARTITION should be kept 0.
 *
 * \showinitializer
 */
#define _MULTI_PARTITION  0

/** Allow erase of blocks of sectors
 *
 *  This is a device driver function "below" the file system. For use with
 *  \ref intro_sec "weAutSys" it should be kept disabled to avoid conflicts
 *  with the runtime's driver functions.
 *
 *  Note: Files and directories are erased by \ref f_unlink().
 *
 *  0: disable / 1: enable sectors erase.
 *
 *  If set to 1 the \ref CTRL_ERASE_SECTOR command should be added to the
 *  \ref disk_ioctl function.
 *
 * \showinitializer
 */
#define _USE_ERASE 0


//---- System Configurations

/** Word access
 *
 *  values: 0: Byte-by-byte access / 1: Word access.
 *
 *  ChaN says:  When the byte order on the memory is big-endian or address
 *  miss-aligned word access results incorrect behavior, the _WORD_ACCESS
 *  must be set to 0.
 *
 *  Hint (A.W.): In all other cases (like ref intro_sec "weAutSys",
 *  ref intro_secH "weAut_01", ATmega, GCC) it should be left 1 as the
 *  C and machine code generated by \c  _WORD_ACCESS == 0 is not optimal (to
 *  put it mildly). <br />
 *  Hint2: The name of this configuration option is a bit misleading. It
 *  concerns not really the bus width (8, 16, 32, 64) of the computer
 *  architecture used but more (or only) its endianess.
 *  \showinitializer
 */
#define _WORD_ACCESS  1

/** @} */
/** @} */ // osSWinc
#endif /* _FFCONFIG */

