/**
 * \addtogroup osSWinc
 * @{
 * \addtogroup uipApp
 * @{
 * \defgroup resolv DNS resolver
 * @{
 * \details
 * The uIP DNS resolver functions are used to lookup a hostname and map it to
 * an IP address. It maintains a list of resolved hostnames that can be
 * queried with the resolv_lookup() function. New hostnames can be resolved
 * using the resolv_query() function.
 *
 * When a hostname has been resolved (or found to be non-existent),
 * the resolver code calls a callback function called resolv_found()
 * that must be implemented by the module that uses the resolver.
 */


/**
 * \file resolv.h
 *
 * DNS resolver definitions
 *
 *  Adam Dunkels' original copyright notice <br /> &nbsp;
 *  Copyright (c) 2002-2003, Adam Dunkels.
 *  All rights reserved. <br />
 *  (contained in this file) still holds. For modifications:
 *
 *  <br /> &nbsp;
 * \ref copyLicNote " Copyright &copy; " 2011 Albrecht Weinert, Bochum
 * \author Adam Dunkels <adam@dunkels.com>
 * \author  Albrecht Weinert  &nbsp; <<a href="http://a-weinert.de">a-weinert.de</a>>
 * $Revision: 2 $
 * $Date: 2017-01-25 16:49:03 +0000 (Wed, 25 Jan 2017) $
 */

/* Copyright (c) 2002-2003, Adam Dunkels.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This file is part of the uIP TCP/IP stack.
 */
#ifndef __RESOLV_H__
#define __RESOLV_H__

#include "uip/uipopt.h"

/** The maximum number of DNS retries
 *
 *  \showinitializer
 */
#define MAX_DNS_RETRIES 8

/** uIP event function for the DNS resolver
 *
 *  Must only be called for the correct protocol / port (rport 53).
 */
void resolv_appcall(void);

/** Callback function which is called after hostname look-up
 *
 * This function must be implemented by the module that uses the DNS
 * resolver. It is called when the hostname requested got an IP address
 * assigned by DNS server answer, or when such assignment could not be
 * found.
 *
 * \param name   pointer to the name that was to look up
 * \param ipaddr pointer to a 4-byte array containing the IP address of the
 *        hostname, or NULL if the hostname could not be found.
 */
void resolv_found(char *name, uint16_t *ipaddr);


/** Configure which DNS server to use for queries
 *
 *  \note \ref intro_sec "weAutSys" configuration can hold up to two DNS
 *  server IPs usually got by \ref dhcpclient "DHCP".
 *
 * \param dnsserver  a pointer to a 4-byte representation of the
 *                   DNS server's IP address
 */
void resolv_conf(uint16_t *dnsserver);

/** Obtain the currently configured DNS server
 *
 *  \return A pointer to a 4-byte representation of the IP address of
 *  the currently configured DNS server or NULL if no DNS server has
 *  been configured.
 */
uint16_t * resolv_getserver(void);

/** Initialise the resolver  */
void resolv_init(void);

/** Look up a hostname in the array of known hostnames
 *
 * \note This function only looks in the internal array of known hostnames.
 * It does not send out a query for the hostname, even if none  was found.
 * The function \ref resolv_query() can be used to send such query.
 *
 * \return A pointer to a 4-byte representation of the hostname's IP
 * address, or NULL if the hostname was not found in the array of
 * hostnames.
 */
uint16_t * resolv_lookup(char *name);

/** Queues a name so that a question for the name will be sent out
 *
 * \param name The hostname that is to be queried.
 */
void resolv_query(char *name);

#endif /* __RESOLV_H__ */

/** @} */
/** @} */
/** @} */ //osSWinc
