// Copyright (c) 2011 Albrecht Weinert, Bochum
// persist.h 2 2017-01-25 16:49:03Z albrecht


/**
 * \addtogroup weAutSys
 * @{
 *
 * \addtogroup utilHelp
 * @{
 *
 * \addtogroup utilPersist
 * @{
 *
 * \file persist.h
 *
 *  \ref intro_sec "weAutSys" (weAut_01) utility / library functions for persistent storage
 *
 *  This file contains the definitions for \ref intro_sec "weAutSys"
 *  (weAut_01) utility / library functions for persistent storage. This is
 *  system software and must not be modified for user or application programs.
 *
 *
 * This file is part of \ref intro_sec "weAutSys" &nbsp;
 *          <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2011 Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp;
 *                            <<a href="http://a-weinert.de">a-weinert.de</a>>
 * 2
 * 25.01.2017
 */
//  SVN last change by A. Weinert
/** @} */ // Doxygen addtogroup  utilHelp
/** @} */ // Doxygen defgroup utilPersit

#ifndef PERSIST_H_
#define PERSIST_H_

#include <stdint.h>
#include "pt/pt.h"
#include "we-aut_sys/network.h"

/**
 * \addtogroup utilHelp
 * @{
 *
 * \addtogroup utilPersist
 * @{
 */

/** Read one EEPROM cell to RAM
 *
 *  The operation is a bit slow (compared to RAM access) but the function
 *  returns immediately.
 *
 *  The function fails if another EEPROM (or Flash write) operation is
 *  running. It must, of course, never ever be used spin waiting for success.
 *
 * \param eeAddr the EEPROM address to read (0 .. 4095)
 * \param readValue pointer to the (RAM) byte to put the value read in
 * \return 0: fail; 1: success
 * \sa eeWriteByte
 * \sa eeReadBytes
 */
uint8_t eeReadByte(uint8_t * readValue, uint16_t eeAddr);

/** Read EEPROM cells
 *
 *  This function reads a number of consecutive EEPROM cells. Reading EEPROM
 *  is four to five times slower than RAM access.
 *
 *  The function fails and returns immediately if another EEPROM (or Flash
 *  write) operation is running. It must, of course, never ever be used
 *  spin waiting for success.
 *
 * \param eeAddr the EEPROM address to read (0 .. 4095)
 * \param dest   the RAM address to put the EEPROM data to
 * \param bufferLength the number of bytes to handle 1..255
 * \return the number of bytes transfered (may be shorter than bufferLenght
 *         or 0 in case of failure or inadequate parameters
 * \sa eeReadByte
 */
uint8_t eeReadBytes(uint16_t eeAddr, uint8_t * dest,  uint8_t bufferLength);

/** Write one EEPROM cell
 *
 *  This function writes the EEPROM cell. If the (slow and wearing) write is
 *  necessary it starts the write procedure for the new changed content.
 *  The operation is quite slow (compared to RAM access) but the function
 *  returns immediately either after failure or after just starting the
 *  (3 ms) EEPROM write procedure.
 *
 *  The function fails if another EEPROM (or Flash write) operation is
 *  running. It should not be used by application software and it must,
 *  of course, never ever be called spin waiting for success.
 *
 * \param eeAddr the EEPROM address to (read and) write
 * \param newValue  the (new) value to write in the EEPROM cell[ eeAddr]
 * \return 0: fail; 1: success
 * \sa eeReadByte
 */
uint8_t eeWriteByte(uint16_t eeAddr, uint8_t newValue);


/** The data structure for an EEPROM bulk operation thread
 *
 *  A structure of this type holds all data for an EEPROM write operation on
 *  1..255 continuous bytes. It will be passed as parameter to the thread
 *  function \ref eeOperationThread.
 *
 *  \sa initSetAppThread(struct mThr_data_t *, p2ptFun)
 */
typedef struct {

   pt_t pt; //!< The (raw) protothread data structure

/** The EEPROM address to start the operation
 *
 *  It has to be set to the first destination address in EEPROM before the
 *  thread function is called the first time. <br />
 *  It will be incremented after each step.
 */
   uint16_t eeAddress;

/** Pointer to the buffer of data to be written
 *
 *  The new data for the EEPROPM are taken from here. <br />
 *  This pointer will be incremented after every step.
 *
 *  For the thread function \ref eeOperationThread this is an address in RAM.
 */
   uint8_t * writeBuffer;

/** Number of bytes to write
 *
 *  It has to be set accordingly before the thread function is called the
 *  first time. This value will be decremented after every step.<br />
 *  0, of course, means no (further) operations pending.
 */
   uint8_t noOfWrite;
} eeOp_data_t;

/** The data for an EEPROM bulk operation thread
 *
 *  It is quite possible to have more than one active protothread (function
 *  \ref eeOperationThread) at the same time. In that case each must use its
 *  own \ref eeOp_data_t structure.
 *
 *  On the other hand it seems clever to restrict EEPROM access to just
 *  one thread at one time. In that case they all can use the same
 *  \ref eeOp_data_t structure, optionally using noOfWrite as a signal for
 *  availability. This \ref  eeOpData are used by \ref intro_sec "weAutSys" system software
 *  at initialisation time only. Application software is free to use it
 *  afterwards (i.e. in any thread except \ref appInitThreadF).
 */
extern eeOp_data_t eeOpData;

/** Initialise an EEPROM operation thread
 *
 *  Before scheduling the EEPROM operation thread for first start of a
 *  bulk operation it's thread data (eeOpData) must be initialised.
 *
 *  Calling this or another initialising function while a thread using
 *  the structure (eeOpData) is operating will have unwanted if not
 *  catastrophic results.
 *
 * \param eeOpData the thread's data
 * \param eeAddr the EEPROM address to (read and) write
 * \param writeBuffer the source of the new EEPROM data
 * \param bufferLength the number of bytes to handle
 * \return the number of bytes to be handled by the thread
 *         (may be shorter than bufferL)
 * \sa initEEthreadRead
 */
uint8_t initEEthreadWrite(eeOp_data_t * eeOpData, uint16_t eeAddr,
                          uint8_t * writeBuffer,  uint8_t bufferLength);


/** The EEPROM (bulk) write system thread; the thread function
 *
 *  According to the task set in eeOpData this thread will transfer data
 *  from RAM to EEPROM.
 *
 *  At first schedule(s) this thread blocks as long as it has not gained
 *  access to the EEPROM.
 *
 *  After the operation has begun it will yield after every step requiring
 *  an EEPROM write and after every eight steps without the need to modify
 *  the EEPROM content.
 *
 *  As one EEPROM write requires 3.4 ms, it could be considered to schedule
 *  every forth time in a 1 ms cycle or at lower frequency.<br />
 *
 *  This thread is best scheduled by \code
 *
 *  initEEthreadWrite(&eeOp_data, .....);
 *  PT_WAIT_THREAD(callingParentThread, eeOperationThread(&eeOpData));
 *
 *  \endcode
 *
 * \param eeOpData pointer to the data of the EEPROM operation
 * \return PT_WAITING while waiting for EEPROM  ready on first schedule(s);
 *         PT_YIELDED while waiting for EEPROM ready after a modify step;
 *         PT_YIELDED after every 8th read only step;
 *         PT_ENDED when ready
 */
ptfnct_t eeOperationThread(eeOp_data_t * eeOpData);


/** EEPROM address of (address of) EEPROM configuration data
 *
 *  This is the address of two consecutive bytes in EEPROM that hold the
 *  address of this devices EEPROM configuration data.
 *
 *  The rationale of this indirection is that that the entry pointed to by
 *  this address will have to be changed only if the writing of configuration
 *  data fails due to EEPROM wear out (after > 100000 modifies).
 *
 *  \showinitializer
 *  \sa DEFAULT_EEPROM_CONF_ADD
 */
#define EEPROM_POINTER2_EE_CONF (EEP_SIZE - 64)


/** Default address of EEPROM configuration data
 *
 *  This is the (fresh board default) value for the address of this
 *  devices EEPROM configuration data.
 *
 *  The value might be changed in future software releases to use other
 *  EEPROM locations on (then) older boards.
 *
 *  Hint: This value has to be used in (C-) sources and tool (avr-objcopy)
 *  settings respectively parameters to generate an EEPROM .hex file with
 *  individualized board configurations.
 *
 *  Hint 2: There is a (until April 2013 at least) avrdude bug that
 *  erases all EEPROM cells below the first address of a small EEPROM .hex
 *  file. As long as this problem persists \ref DEFAULT_EEPROM_CONF_ADD should
 *  be quite low and must be lower than \ref EEPROM_POINTER2_EE_CONF.
 *
 *  \showinitializer
 *  \sa EEPROM_POINTER2_EE_CONF
 */
#define DEFAULT_EEPROM_CONF_ADD 64
// internal til 07.02.12 64 til 08.04.13 128


/** Address of EEPROM configuration data
 *
 *  This is the address of this device's EEPROM configuration data.
 *
 *  This EEPROM address may be in the range 64 .. (EEPROMsize - 64).<br />
 *  It is read at start-up from EEPROM[ \ref EEPROM_POINTER2_EE_CONF ]. If
 *  outside said rage a default configuration will be set up at EEPROM
 *  address 64.
 *
 *  \sa conf_data_t
 *  \sa EE_CONF_ADD
 */
extern uint16_t  eeConfigAdd;

/** The EEPROM address of a configuration element
 *
 *  This macro calculates the EEPROM address of an element in a
 *  \ref conf_data_t structure. This is done without without any precondition
 *  checks. The most important condition is \ref eeConfigAdd being not NULL.
 *
 *  \param elem the name of a \ref conf_data_t element
 *  \return     its EEPROM address in an \ref eeConfigAdd based
 *              \ref conf_data_t structure
 */
#define EE_CONF_ADD(elem) ((uint16_t)&(((conf_data_t*)eeConfigAdd)->elem))

/** @} */ // Doxygen addtogroup utilPersist
/** @} */ // Doxygen defgroup utilHelp


//-- Initialisation functions ------------------------
/**
 * \addtogroup llSySer
 * @{
 * \addtogroup  sysInit
 * @{
 */

/** The device's basic configuration data type
 *
 *  A structure of this type holds basic configuration data specific to the
 *  individual device. To survive power off one copy of these data is to be
 *  held in EEPROM at address
 *  \ref eeConfigAdd = EEPROM[ \ref EEPROM_POINTER2_EE_CONF ].
 */
typedef struct {

   eth_addr_t macAddress; //!< the MAC address

/** The (magic) network configuration number
 *
 *  This is a kind of major revision number for the configuration data
 *  structures and types. It must neither be 0 nor FF. It has to be changed
 *  whenever a software change would render previous EEPROM data invalid.
 *
 *  The value found in the EEPROM configuration data has to checked against
 *  that in \ref defaultTypeConfData; this is done by \ref persistInit().
 */
   uint8_t magicNumber;

/** The IP configuration  */
   ipConf_t ipConf;

} conf_data_t;


/** The device's basic default configuration data
 *
 *  These are the basic configuration data specific to the individual device.
 *  To survive power off one copy of these data is to be held in EEPROM at
 *  address \ref eeConfigAdd = EEPROM[ \ref EEPROM_POINTER2_EE_CONF ].
 */
extern conf_data_t defaultConfData;

/** The device type's basic default configuration data in flash memory
 *
 *  These are the basic default configuration data specific to the device
 *  type. These values typically are used only as long as no other
 *  device specific (individual) values were set in persistent EEPROM
 *  storage.
 *
 *  So this is just the first startup default before commissioning to use.
 */
extern conf_data_t defaultTypeConfData;


/** Initialise persistence / EEPROM handling
 *
 *  This function uses spin waiting for EEPROM (write) operations. It is to be
 *  used in initialisation (after reset) only.
 */
void persistInit(void);



/** @} */ // Doxygen defgroup sysInit


#endif /* PERSIST_H_ */
/** @} */ // Doxygen defgroup utilHelp
/** @} */ // Doxygen defgroup weAutSys
