// Copyright (c) 2011 Albrecht Weinert, Bochum
//enc28j60.c 2 2017-01-25 16:49:03Z albrecht

/**
 * \addtogroup weAutSys
 * @{
 * \addtogroup llSySer
 * @{
 * \addtogroup ethCom
 * @{
 *
 *  \file enc28j60.c
 *  weAutSys' (weAut_01's)  28J60 Ethernet driver
 *
 *  This file contains the implementation of (low level) system calls and
 *  services for the \ref intro_sec "weAutSys"' (weAut_01's)
 *  ENC28J60 Ethernet driver. <br />
 *  This source is for the ENC28J60 on weAut_01. For other platforms omit or
 *  replace it.
 *
 *  This is system software and must not be modified for user or
 *  application programs.
 *
 *  See also the paragraph \ref ethCom_sec "Ethernet communications".
 *
 * This file is part of weAutSys &nbsp;
 *          <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 *
 *  <br /> &nbsp;
 * \ref copyLicNote " Copyright &copy; " 2011 Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp; <<a href="http://a-weinert.de">a-weinert.de</a>>
 * 2
 * 25.01.2017
 */
//  SVN last change by A. Weinert


/** @} */ // doxygen addtogroup ethCom
/** @} */ // doxygen addtogroup llSySer
/** @} */ // doxygen addtogroup weAutSys


#include "we-aut_sys/common.h"
#include "we-aut_sys/enc28j60.h"
#include "we-aut_sys/spi.h"

#include <stdint.h>
#include <util/delay_basic.h>

uint8_t currentBank; // The current bank of the Registers

#if DEBUG_ETH > 1  // for test phase debug only  ----------------------------
# include "we-aut_sys/log_streams.h"
# include "we-aut_sys/streams.h"
# include "we-aut_sys/utils.h"
# include "we-aut_sys/system.h"
INFLASH(char const encContReg[]) = "  ENC contReg(";
INFLASH(char const enccEthReg[]) = "  ENC cEthReg(";
INFLASH(char const encPhyReg[])  = "  ENC PhyReg(";
INFLASH(char const readFrmReg[]) = "  -r> ";
INFLASH(char const writeToReg[]) = "  <w- ";
INFLASH(char const encBankChg[]) = "  ENC Bank (ECON1) changed to: ";

void logContReg(uint8_t reg){
   bufLogHex_P(encContReg, currentBank);
   bufLogPutC('|');
   bufLogHex(reg);
   bufLogPut2C(')', ' ');
} // logContReg(uint8_t)

void logCcEthReg(uint8_t reg){
   bufLogHex_P(enccEthReg, currentBank);
   bufLogPutC('|');
   bufLogHex(reg);
   bufLogPut2C(')',' ');
} // logCcEthReg(uint8_t)
#endif             // for test phase debug only  ----------------------------


#define IS_ETH_REG (currentBank <= 0x20 || regAdd >= KEY_REGISTERS \
      || (currentBank == 0x60 && regAdd > 5  && regAdd != 0x0A))

/* Read a control register */
uint8_t readControlRegister(uint8_t regAdd) {
   regAdd = setBank(regAdd); // switch Bank if necessary
   selectEnc();
   // Transmit opcode for read register + register address/
   uint8_t data = tranceiveEnc2(RCR_CMD | regAdd, 0);
#if DEBUG_ETH > 2
   const uint8_t dummy = data;
#endif

   const uint8_t mamireg = !IS_ETH_REG;
   if(mamireg)  data = tranceiveEnc(0); // read again cause 1st was dummy
   deSelectEnc();

#if DEBUG_ETH > 2
   /// replace std and printf
   //  while (outSpace(stdout) < 50);
   if(mamireg) {
      logContReg(regAdd);
      bufLogHex_P(readFrmReg, data);
      bufLogPut2C(',',' ');
      bufLogHex(dummy);
      bufLogPut2C(' ','\n');
   } else {
      logCcEthReg(regAdd);
      bufLogHex_P(readFrmReg, data);
      bufLogPut2C(' ','\n');
   }
#endif
   return data;
} // readControlRegister


/* Write data to a control register */
void writeControlRegister(uint8_t regAdd, uint8_t data){
   regAdd = setBank(regAdd);  //  switch Bank if necessary
   selectEnc();
   tranceiveEnc2(WCR_CMD | regAdd, data); // opcode write-ctrl-register | add
   deSelectEnc();

#if DEBUG_ETH > 2
   /// replace std and printf
  // while (outSpace(stdout) < 50);
   logContReg(regAdd);
   bufLogHex_P(writeToReg, data);
   bufLogPut2C(' ','\n');
#endif
} // writeControlRegister


/* Set bits in a control register */
void setBitfield(uint8_t regAdd, uint8_t data){
   regAdd = setBank(regAdd);  //  switch Bank if necessary
   selectEnc();
   if(IS_ETH_REG) {
      tranceiveEnc2(BFS_CMD | regAdd, data);
   } else {
      tranceiveEnc(RCR_CMD | regAdd);
      tranceiveEnc(0);
      const uint8_t ri = tranceiveEnc(0);
      data =  ri | data;
      if (ri != data) {
         deSelectEnc();
         selectEnc();
         tranceiveEnc2(WCR_CMD | regAdd, data);
      }
   }
   deSelectEnc();
#if DEBUG_ETH > 2
   /// replace std and printf
   // while (outSpace(stdout) < 50);
   logContReg(regAdd);
   bufLogPut2C('s','=');
   bufLogHex(data);
   bufLogPut2C(' ','\n');
   //p rintf("  ENC contReg(0x%02x|0x%02x)  |s= 0x%02x  \n",
     //                                          currentBank, regAdd  , data);
#endif
} // setBitfield


/* Clear bits in a control register */
void clearBitfield(uint8_t regAdd, uint8_t data){
   regAdd = setBank(regAdd);
   selectEnc();
   if(IS_ETH_REG) {
      tranceiveEnc2(BFC_CMD | regAdd, data);
   } else {
      tranceiveEnc(RCR_CMD | regAdd);  ///
      tranceiveEnc(0);
      const uint8_t ri = tranceiveEnc(0);
      data =  ri | ~data;
      if (ri != data) {
         deSelectEnc();
         selectEnc();
         tranceiveEnc2(WCR_CMD | regAdd, data);
      }
   }
   deSelectEnc();
#if DEBUG_ETH > 2
   /// replace std and printf
   // while (outSpace(stdout) < 50);
   logContReg(regAdd);
   bufLogPut2C('&', '~');
   bufLogHex(data);
   bufLogPut2C(' ', '\n');
   //  p rintf("  ENC contReg(0x%02x|0x%02x)  &~c 0x%02x  \n",  currentBank, regAdd  , data);
#endif
} // clearBitfield


#if DEBUG_ETH > 1
//                             0123456 89.123456789.123
INFLASH(char const encBufPtMes[] ) = "  ENC buffer mem <w- 0x";
INFLASH(char const encBufRdMes[] ) = "  ENC buffer r> [";
//                             0123456 89.123456789
INFLASH(char const encBufWrMes[] ) = "  ENC buffer <w [";

INFLASH(char const encMACset[]    ) = "  ENC set MAC : ";
INFLASH(char const encHWtrbl[]    ) = "  ENC HW trouble";
INFLASH(char const encMACgot[]    ) = "  ENC got MAC : ";
#endif


/* Read n bytes from ENC28J60's memory buffer
 *
 *  This function reads a sequence of bytes from the dual port RAM byte
 *  indirectly addressed by the \ref ERXRDPTL "ERXRDPT" control registers.
 */
void readBufferMemory(uint8_t * dest, uint16_t n){
#if DEBUG_ETH > 2
   bufLogDec_P(encBufRdMes, n);  bufLogPut2C(' ','\n');
#endif
   if (!n) return;

   selectEnc();
   tranceiveEnc(RBM_CMD); // transmit opcode for read buffer
   while(1) {
      const uint8_t leByte = tranceiveEnc(0); // send (dummy= 0 to receive
      *dest = leByte;
#if DEBUG_ETH > 3
      char tmp[] = "   ";
      twoHexs(tmp, leByte);
      //  while (uartOutSpace() < 3);
      bufLogTxt(tmp, 3);
#endif
      if (!--n) break;
      ++dest;
   }
   deSelectEnc();
} // readBufferMemory


/* Index of the actual receive packet in enc's dual port memory
 *
 * If at least one package has been received this will be the index to its
 * begin. Most of the time this should be in accordance with
 * \ref ERXRDPTL "ERXRDPT"
 */
uint16_t actPackInd;


/* Write data to output buffer */
void putBufferMemory(uint8_t data){
  selectEnc();
  tranceiveEnc2(WBM_CMD, data);
  deSelectEnc();
#if DEBUG_ETH > 3
  bufLogHex_P(encBufPtMes, data);
#endif
} // putBufferMemory

/* Write n bytes to ENC28J60's memory buffer
 *
 *  \param source  pointer to RAM data to be written into the buffer memory
 *  \param n     number of bytes to be written
 */
void writeBufferMemory(uint8_t * source, uint16_t n){
#if DEBUG_ETH > 2
   bufLogDec_P(encBufWrMes, n);  bufLogPut2C(' ','\n');
#endif

   if (!n) return;
   selectEnc();
   tranceiveEnc(WBM_CMD);
   while (1) {
      const uint8_t leByte =  *source;
      tranceiveEnc(leByte);
#if DEBUG_ETH > 2
      char tmp[] = "   ";
      twoHexs(tmp, leByte);
      // while (uartOutSpace() < 3);
      bufLogTxt(tmp, 3);
#endif
      if (!--n) break;
      ++source;
   }
   deSelectEnc();
} // writeBufferMemory



/* Read Physical register  */
uint16_t readPhysicalRegister(uint8_t regAdd){
  // PHY register address into the MIREGADR register first
  writeControlRegister(MIREGADR, regAdd);

  // write MIIRD bit to perform reading from register into  MICMD
  writeControlRegister(MICMD, (1<<MIIRD));

  // ENC's data sheet says wait 10,24 s
  // wait until BUSY bit is reset to 0 in MISTAT
  // should not be interrupted by any other operation on the controller
  uint8_t mxWait = 4; // allow only 40 s for TEST
  do {
     if (!(--mxWait)) {
        spi2Errors |= 0x88; // new ENC busy error
        if(nicIsSelected()) spi2Errors |=  0x20;  \
        if(memCardIsSelected()) spi2Errors |=  0x40; \
        break;  // just stop waiting
     }
     _delay_loop_1(67); // 10s     ((254 = 38,4s at 20Mhz))
  } while( readControlRegister(MISTAT) & (1<<BUSY));

  // write 0 to MICMD register to reset the read bit
  writeControlRegister(MICMD, 0);
  ucnt16_t ret;
  ret.v8[1] = readControlRegister(MIRDH);
  ret.v8[0] = readControlRegister(MIRDL);

#if DEBUG_ETH > 2
  /// replace std and printf
  //while (outSpace(stdout) < 40);
  bufLogHex_P(encPhyReg, regAdd);
  bufLogTxt_P(readFrmReg);
  bufLogHex(ret.v8[1]); bufLogHex(ret.v8[0]);
  bufLogPut2C(' ','\n');
  // p rintf("  ENC PhyReg(0x%02x) -r> 0x%04x  \n", regAdd, ret.v16);
#endif

  return ret.v16;
} // readPhysicalRegister


/* Write data to a physical register */
void writePhysicalRegister(uint8_t regAdd, uint16_t data){
  writeControlRegister(MIREGADR, regAdd);    // PHY register address
  writeControlRegister(MIWRL, LBYTE(data)); // write data in MIWRL and MIWRH
  writeControlRegister(MIWRH, HBYTE(data)); //will set the BUSY flag in MISTAT

  // wait until BUSY bit is reset to 0 in MISTAT
  // should not be interrupted by any other operation on the controller
  // todo why not wait uncond before like read TEST !!

  uint8_t mxWait = 4; // allow only 40 s for TEST
  do {
     if (!(--mxWait)) {
        spi2Errors |= 0x88; // new ENC busy error
        if(nicIsSelected()) spi2Errors |=  0x20;  \
        if(memCardIsSelected()) spi2Errors |=  0x40; \
        break;  // just stop waiting
     }
     _delay_loop_1(67); // 10s     ((254 = 38,4s at 20Mhz))
  } while( readControlRegister(MISTAT) & (1<<BUSY));

//  while(( readControlRegister(MISTAT) & (1<<BUSY))){ // spin wait
  //   _delay_loop_1(67); // 10s 254 = 38,4s at 20Mhz
  //}

#if DEBUG_ETH > 2
  /// replace std and printf
 /// while (outSpace(stdout) < 40);
  bufLogHex_P(encPhyReg, regAdd);
  bufLogTxt_P(writeToReg);
  bufLogHex(HBYTE(data)); bufLogHex(LBYTE(data));
  bufLogPut2C(' ','\n');
  // p rintf("  ENC PhyReg(0x%02x) <w- 0x%04x  \n", regAdd, data);
#endif

} // writePhysicalRegister

/* Set a register bank (in ECON1) if necessary for the given register */
uint8_t setBank(uint8_t regAdd){
   const uint8_t pureAdd = regAdd & REG_MASK;
   if (pureAdd >= KEY_REGISTERS) return pureAdd;
   regAdd = regAdd & REG_BANK_MASK;
   if (regAdd == currentBank) return pureAdd;

   uint8_t bits = ~regAdd & currentBank; // bits to clear

#if DEBUG_ETH > 2
   /// replace std and printf
  // while (outSpace(stdout) < 50);
  bufLogTxt_P(encBankChg);
  bufLogDecB(regAdd >> 5);
  bufLogPut2C(';', ' ');
  bufLogHex(regAdd);
  bufLogPut2C('<', ' ');
  bufLogHex(currentBank);
  bufLogPut2C(' ','\n');
 // p rintf("  ENC Bank (ECON1) changed to: %d ; %02x < %02x  \n",
   //                            regAdd >> 5, regAdd, currentBank);
#endif

   if (bits) {
      selectEnc();
      tranceiveEnc(BFC_CMD | ECON1);
      tranceiveEnc(bits >> 5);
      deSelectEnc();
   }
   bits = regAdd & ~currentBank; // bits to set
   if (bits) {
      selectEnc();
      tranceiveEnc(BFS_CMD  | ECON1);
      tranceiveEnc(bits >> 5);
      deSelectEnc();
   }
   currentBank = regAdd;
#if DEBUG_ETH > 2
   readControlRegister(ECON1);
#endif

   return pureAdd;
} // set_bank


/* Check the link state
 * \return 0 if the link is down;  1 if continuously up 2 if up but was down
 */
uint8_t linkState(){
   if (readPhysicalRegister(PHSTAT2) & (1<<LSTAT)) { // link currently up
      networkNotReady &= ~LNC_IDN; // clear link down bit
      if (readPhysicalRegister(PHSTAT1) & (1<<LLSTAT)) { // was conti. up
         networkNotReady &= ~LNC_WDN; // clear link was down bit
         return 1;
      } // link was continuously up
      networkNotReady |= LNC_WDN; // set link was down bit
      return 2;
   } // link currently up

   // to here if link currently down
   if (networkNotReady & LNC_IDN) { // is already set as down
      networkNotReady |= LNC_WDN; // set link was down bit (now) too
   } else {
      networkNotReady &=~ LNC_WDN; // clear link was down bit (as we thought)
      networkNotReady |= LNC_IDN;
   }
   return 0;
} // link_state


//---  Initialisation functions -------------------------------

void encReset(void){
  networkNotReady = (networkNotReady & (ENC_NRD | ENC_HWP | STC_MSK))
         | ENC_NRD | ENC_HWP;  // bit 7 : ENC reset bit 6 : HW clock trouble
  selectEnc();
  tranceiveEnc(SYSRST_CMD);
  deSelectEnc();
} // encReset


/* Read the current MAC address */
void encGetMacAdd(eth_addr_t * mac){
  mac->addr[0] = readControlRegister(MAADR1);
  mac->addr[1] = readControlRegister(MAADR2);
  mac->addr[2] = readControlRegister(MAADR3);
  mac->addr[3] = readControlRegister(MAADR4);
  mac->addr[4] = readControlRegister(MAADR5);
  mac->addr[5] = readControlRegister(MAADR6);

#if DEBUG_ETH > 1
  // while (uartOutSpace() < 40);
  bufLogTxt_P(encMACgot);
  char tmp[17];
  formMacAdd(tmp, mac);
  bufLogTxt(tmp, 17);
  bufLogPut2C(' ', '\n'); // 1 blank 1 LF
#endif
} // encGetMacAdd


/* Set the drivers MAC address  */
uint8_t encSetMacAdd(eth_addr_t * mac){
  writeControlRegister(MAADR1, mac->addr[0]);
  writeControlRegister(MAADR2, mac->addr[1]);
  writeControlRegister(MAADR3, mac->addr[2]);
  writeControlRegister(MAADR4, mac->addr[3]);
  writeControlRegister(MAADR5, mac->addr[4]);
  writeControlRegister(MAADR6, mac->addr[5]);

  uint8_t retV = 0;

  if (readControlRegister(MAADR6) != mac->addr[5]) retV = 1;
  if (readControlRegister(MAADR1) != mac->addr[0]) ++retV;
  if (readControlRegister(MAADR4) != mac->addr[3]) ++retV;
  if (readControlRegister(MAADR2) != mac->addr[1]) ++retV;
  if (readControlRegister(MAADR3) != mac->addr[2]) ++retV;
  if (readControlRegister(MAADR5) != mac->addr[4]) ++retV;

#if DEBUG_ETH >= 2
  if (bufLogStreamsOutSpace(NULL) >= 38 ){
     bufLogTxt_P(encMACset);
     char tmp[17];
     formMacAdd(tmp, mac);
     bufLogTxt(tmp, 17);
     if (retV) {
        bufLogTxt_P(encHWtrbl);
        bufLogPut2C(' ', '0' + retV); // 1 blank number of errors
     }
     bufLogPut2C(' ', '\n'); // 1 blank 1 LF
  }
#endif
  return retV;
} // encSetMacAdd


/* Set the size of the receive and transmit buffers */
inline void encSetBufferLimits(){
   writeControlRegister(ECON1, 0xC0); // Tr & Rx reset & disable, bank 0

  // Transmission buffer start (1A0E, even; one after receive buffer is OK)
  writeControlRegister(ETXSTL, LBYTE(TXBUF_STRT));
  writeControlRegister(ETXSTH, HBYTE(TXBUF_STRT));
  // transmit buffer end ETXNDL/H is dynamically assigned before send
  writeControlRegister(EWRPTL, LBYTE(TXBUF_STRT));
  writeControlRegister(EWRPTH, HBYTE(TXBUF_STRT)); // write pointer
  

  // Set start of receive buffer; always starts from 00:00 due to ENC bug
  writeControlRegister(ERXSTL, 0);
  writeControlRegister(ERXSTH, 0);
  actPackInd = 0; // will be the actual (= next to receive) index
  writeControlRegister(ERXRDPTL, 0);
  writeControlRegister(ERXRDPTH, 0);
  
  writeControlRegister(ERDPTL, 0);
  writeControlRegister(ERDPTH, 0);  // read pointer

  // Receive buffer end (1A0D, odd)
  writeControlRegister(ERXNDL, LBYTE(RXBUF_END));
  writeControlRegister(ERXNDH, HBYTE(RXBUF_END));

  // clearBitfield(ECON1, 0xC0); // remove resets from TR & RX
} // encSetBufferLimits


void encEnablePowersave(void){
  setBitfield(ECON2, (1<<PWRSV));
  networkNotReady |= (1<<PWRSV);
} // enablePowersave


void encDisablePowersave(void){
  clearBitfield(ECON2, (1<<PWRSV));
  networkNotReady &= ~(1<<PWRSV);
} // disablePowersave

/* Transmission status vector  */
uint8_t transmitStatVec[7];


#if DEBUG_ETH >= 2
  //                                0123456789.123456789.123456789
INFLASH(char const initENCrp[]) = "\n  ENC initialisation * *  \n";
INFLASH(char const initdENCrp[]) = "  ENC initialised  Rev.0x";
INFLASH(char const noIniENCrp[]) = "  ENC not init'd state 0x";
INFLASH(char const readENCcnr[]) = "  ENC read control reg ";
INFLASH(char const readENCphr[]) = "  ENC read physicl reg ";
INFLASH(char const ENCcnrECON2[]) = "ECON2";
INFLASH(char const ENCcnrECON1[]) = "ECON1";
INFLASH(char const ENCphrPHID1[]) = "PHID1";
INFLASH(char const ENCphrPHID2[]) = "PHID2";

#endif


/* Initialise the ENC Controller with default values */
void encInit(){
#if DEBUG_ETH >= 2
  bufLogTxt_P(initENCrp);
#endif

  setBitfield(ECON1, SPI_BANK_MASK); // force consistent
  currentBank = REG_BANK_MASK;      // bank select

  if (networkNotReady & ENC_PWS) { // power save or clock not ready
     clearBitfield(ECON2, (1<<PWRSV));
     networkNotReady = (networkNotReady & ~ENC_PWS) | (ENC_NRD | ENC_HWP);
#if DEBUG_ETH >= 2
     bufLogHex_P(noIniENCrp, networkNotReady);
     bufLogPut2C(' ','\n');
#endif
     return; // no success ENC still not working
  }
  if (encSetMacAdd(&actMACadd)) { //  setMAC address according to uIP
     networkNotReady |= (ENC_NRD | ENC_HWP);
#if DEBUG_ETH >= 2
     bufLogHex_P(noIniENCrp, networkNotReady);
     bufLogPut2C(' ','\n');
#endif
     return; // no success ENC still not working
  }

  encSetBufferLimits();    // set size of memory buffer

  // set packet filter to OR (bit 6 = 0) all packets to us = our MAC
  // (bit7 = 1) and all broadcast (ARP e.g.) without further filtering
  // (bit0 = 1) and reject all packets with wrong CRC (bit5 = 1)
  writeControlRegister(ERXFCON, 0xA1);

#if DEBUG_ETH >= 2
  bufLogTThex_P(readENCcnr, ENCcnrECON1, readControlRegister(ECON1));
#endif

  setBitfield(ECON2, (1<<AUTOINC)); // set AUTOINC should always be

#if DEBUG_ETH >= 2
  bufLogTThex_P(readENCcnr, ENCcnrECON2, readControlRegister(ECON2));
  bufLogTThex_P(readENCcnr, ENCcnrECON1, readControlRegister(ECON1));
  // test read two phy regs if debug
  bufLogTThex_P(readENCphr,  ENCphrPHID1, readPhysicalRegister(PHID1));
  bufLogTThex_P(readENCphr,  ENCphrPHID2, readPhysicalRegister(PHID2));
#endif

  // LED B (ge): Link status; LED B (gn):  transmit; long stretch
  writePhysicalRegister(PHLCON, LA_TX | LB_LSTAT | TLSTRCH);

  // enable MAC receive
  writeControlRegister(MACON1, (1<<MARXEN) | (1<<TXPAUS) | (1<<RXPAUS));
  // some SW bring MAC out of reset by (non existing MACON2 = 0x00 ???
  writeControlRegister(MACON2, 0);


  /* MACON3 setup:
   *   - Expand short frames to 64 Bytes and add CRC.
   *   - Transmit CRC
   *   - Transmit and receive packages of any size
   *   - Check frame lengths
   */
  writeControlRegister(MACON3, EXPSF64 | (1<<TXCRCEN) | (1<<HFRMEN) | (1<<FRMLNEN));

  // set inter-frame gap (non-back-to-back; 0x12, 0xC0 is recommended)
  writeControlRegister(MAIPGL, 0x12);
  writeControlRegister(MAIPGH, 0x0C);
  // set inter-frame gap (back-to-back; 0x12 is the recommended value)
  writeControlRegister(MABBIPG, 0x12);

  // set to UIP_CONF_BUFFER_SIZE + 18
  writeControlRegister(MAMXFLL, LBYTE(UIP_CONF_BUFFER_SIZE + 18));
  writeControlRegister(MAMXFLH, HBYTE(UIP_CONF_BUFFER_SIZE + 18));

  /* MACON4 setup:
   *
   *   - After collision retransmit immediately
   */
  writeControlRegister(MACON4, (1<<NOBKOFF) );

  // prevent automatic loopback of transmitted data
  writePhysicalRegister(PHCON2, (1<<HDLDIS));

  clearBitfield(ECON1, (1<<TXRST) | (1<<RXRST)); // remove resets (if pending)
  setBitfield(ECON1, (1<<RXEN));  // packets received will be put to buffer
  networkNotReady &= STC_MSK; // clear all ENC not ready bits

#if DEBUG_ETH >= 2
  bufLogHex_P(encEthSta, networkNotReady);  bufLogPut2C(' ','\n');
  const uint8_t revid = readControlRegister(EREVID);
  bufLogHex_P(initdENCrp, revid); bufLogPut2C(' ','\n');
#endif

} // encInit

