/**
 *  \file persist.c
 *
 *  weAutSys (weAut_01) utility / library functions for persistent storage
 *
 *  This file contains the implementations of \ref intro_sec "weAutSys"
 *  (weAut_01) utility / library functions for persistent storage. This is
 *  system software and must not be modified for user or application programs.
 *
 *
 * This file is part of weAutSys &nbsp;
 *           <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2011 Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp; <<a href="http://a-weinert.de">a-weinert.de</a>>
 * $Revision: 2 $
 * $Date: 2017-01-25 16:49:03 +0000 (Wed, 25 Jan 2017) $
 */
//  SVN last change by $Author: albrecht $

#include "we-aut_sys/common.h"
#include "we-aut_sys/ll_system.h"
#include "we-aut_sys/persist.h"
#include <avr/io.h>
#include <util/atomic.h>
#include "pt/pt.h"  // for sake of Eclipese only
#include "we-aut_sys/log_streams.h"

#if DEBUG_EEP
# include "we-aut_sys/utils.h"
//                             0123456 89.123456789.123456789.1
INFLASH(char eepReadMes[])  = "  eePROM read  [....] .. \n";
INFLASH(char eepWriteMes[]) = "  eePROM write [....] .. (..) \n";
static char messi[32];
#endif


/* Read one EEPROM cell
 *
 * \param eeAddr the EEPROM address to read (0 .. 4095)
 * \param readValue pointer to byte to put the value read in
 * \return 0: fail; 1: success
 */
uint8_t eeReadByte(uint8_t * readValue, uint16_t eeAddr){
   if((EECR & (1<<EEPE))  // previous operation running or flash prog.
         ||  (SPMCSR & (1<<RWWSB))) return 0; //

   EEAR = eeAddr;
   EECR |= (1<<EERE);   // start read
   * readValue = EEDR;  // after CPU pause 4 clocks read data is in EEDR

#if DEBUG_EEP > 1
   memcpy_P(messi, eepReadMes, 27);
   fourHexs(&messi[16], eeAddr);
   twoHexs(&messi[22], * readValue);
   bufLogTxt(messi, 26);
#endif
   return 1;
}  // eeReadByte(uint8_t *, uint16_t)


/* Read EEPROM cells
 *
 * \param eeAddr the EEPROM address to read (0 .. 4095)
 * \param dest   the RAM address to put the EEPROM data to
 * \param bufferLength the number of bytes to handle 1..255
 * \return the number of bytes transfered (may be shorter than bufferLenght
 *         or 0 in case of failure
 */
uint8_t eeReadBytes(uint16_t eeAddr, uint8_t* dest,  uint8_t bufferLength){
   if (!dest || ! bufferLength || eeAddr >= EEP_SIZE) return 0;
   if (eeAddr + bufferLength > EEP_SIZE) {
      bufferLength = EEP_SIZE - eeAddr + 1;
   }
   if( (EECR & (1<<EEPE)) ||  (SPMCSR & (1<<RWWSB)) ) return 0;
#if DEBUG_EEP > 1
   memcpy_P(messi, eepReadMes, 22);
   fourHexs(&messi[16], eeAddr);
   bufLogTxt(messi, 22);
#endif
   for (uint8_t i = 0; i < bufferLength; ++i) {
      EEAR = eeAddr++;
      EECR |= (1<<EERE);   // start read
      const uint8_t readValue = EEDR;
      dest[i] = readValue;  // after CPU pause 4 clocks read data is in EEDR
#if DEBUG_EEP > 1
      bufLogHex(readValue);
#endif
   }  // for
#if DEBUG_EEP > 1
   bufLogPutC('\n');
#endif

   return bufferLength;
} // eeReadBytes(uint16_t, uint8_t *, uint8_t)


/* Write one EEPROM cell
 *
 * \param eeAddr the EEPROM address to (read and) write
 * \param newValue  the (new) value to write in the EEPROM cell[ eeAddr]
 * \return 0: fail; 1: success
 */
uint8_t eeWriteByte(uint16_t eeAddr, uint8_t newValue){

   if (eeAddr >= EEP_SIZE) return 0;
   if((EECR & (1<<EEPE))  // previous operation running or flash prog.
         || ( SPMCSR & (1<<RWWSB))) return 0; //

   EEAR = eeAddr;
   EECR |= (1<<EERE); // start read
   const uint8_t readValue = EEDR;
#if DEBUG_EEP
   memcpy_P(messi, eepWriteMes, 32);
   messi[2] = 'E';
   fourHexs(&messi[16], eeAddr);
   twoHexs(&messi[22], newValue);
   twoHexs(&messi[26], readValue);
   bufLogTxt(messi, 32);
#endif
   if (readValue == newValue) return 1;
   while(EECR & (1<<EEPE)) ; // should not be necessary ??
   EEDR = newValue;
   ATOMIC_BLOCK(ATOMIC_RESTORESTATE) {
      EECR |= (1<<EEMPE);
      EECR |= (1<<EEPE);
   }  // atomic block
   // while(EECR & (1<<EEPE)) ; // wait for end
   return 1;
} //  eeWriteByte(uint8_t * readValue, uint16_t eeAddr, uint8_t newValue


/* Initialise an EEPROM operation thread for writing
 *
 * \param eeOpData the thread's data
 * \param eeAddr the EEPROM address to (read and) write
 * \param writeBuffer the source of the new EEPROM data
 * \param bufferLength the number of bytes to handle
 * \return the number of bytes to be handled (may be shorter than bufferL
 */
uint8_t initEEthreadWrite(eeOp_data_t * eeOpData, uint16_t eeAddr,
                                uint8_t * writeBuffer, uint8_t bufferLength){
   if (!eeOpData) return 0;
   if (!bufferLength || !writeBuffer || eeAddr >= EEP_SIZE) {
      bufferLength =  0;
   } else  if (eeAddr + bufferLength > EEP_SIZE) {
      bufferLength = EEP_SIZE - eeAddr + 1;
   }
   PT_INIT( &eeOpData->pt );
   eeOpData->writeBuffer = writeBuffer; // may not be NULL
   eeOpData->eeAddress = eeAddr;
   return  eeOpData->noOfWrite = bufferLength;
} // initEEthreadWrite(eeOp_data_t *, ...


/*  The EEPROM (bulk) write system thread, the function
 *
 *  This thread yields as long as it has not gained access to the EEPROM for
 *  the first time. If rescheduling is given up for a yielding thread nothing
 *  has happened on the EEPROM or the read buffer.
 *
 *  After operation begun it will block every time it has to wait
 *  for write completion. As one write requires 3.4 ms, it will be best to
 *  have this thread scheduled every forth time in a 1 ms cycle or at lower
 *  frequency.
 *
 * \param eeOpData the data of the EEPROM operation
 * \return PT_YIELDED while the operation has not yet begun; PT_WAITING while
 *         waiting for EEPROM operation to complete; PT_ENDED when ready
 */
ptfnct_t eeOperationThread(eeOp_data_t * eeOpData){

   PT_BEGIN(&eeOpData->pt);
   PT_WAIT_WHILE(&eeOpData->pt, (EECR & (1<<EEPE)) ||  (SPMCSR & (1<<RWWSB)));

   uint8_t i = 8;
   while (eeOpData->noOfWrite) {
      --eeOpData->noOfWrite;
#if DEBUG_EEP
      const uint16_t eeAddr =   // Atn. uses next line
#endif
      EEAR = eeOpData->eeAddress++;
      EECR |= (1<<EERE); // start read
      const uint8_t readValue = EEDR;
      const uint8_t writeValue =  *(eeOpData->writeBuffer);
      ++eeOpData->writeBuffer;

#if DEBUG_EEP
      memcpy_P(messi, eepWriteMes, 32);
      fourHexs(&messi[16], eeAddr);
      twoHexs(&messi[22], writeValue);
      twoHexs(&messi[26], readValue);
      bufLogTxt(messi, 32);
#endif

      if (writeValue != readValue) {
         EEDR = writeValue;
         ATOMIC_BLOCK(ATOMIC_RESTORESTATE) {
            EECR |= (1<<EEMPE);
            EECR |= (1<<EEPE);
         }  // atomic block

         PT_WAIT_ASYIELD_WHILE(&eeOpData->pt,
                                 (EECR & (1<<EEPE)) || (SPMCSR & (1<<RWWSB)));
         i = 8;
      } else {
         if (! --i) { // after eight
            PT_YIELD_UNTIL(&eeOpData->pt,
                              !((EECR & (1<<EEPE)) ||( SPMCSR & (1<<RWWSB))));
            i = 8;
         } // after eight
      } // else no write
   } // while
   PT_END(&eeOpData->pt)
} // eeOperationThread(eeOp_data_t *)

/* address of EEPROM configuration data  */
uint16_t  eeConfigAdd;


//------------------------------------------------------------------------

/** report text for fresh board: no utilisable EEPROM configuration data */
INFLASH (char txtFreshBoard[])  =
    " * * * fresh weAut_01 board ? \n\n writing default EEPROM conf. ";
//  0123456789.123456789.123456789. 1 23456789.123456789.123456789.12


eeOp_data_t eeOpData;

/** the device's MAC &mdash; startup default only
 *
 *  In the en within one LAN the MACs must, of course, be unique by persistent
 *  EEPROM configuration. For unique DHCP XIDs make bytes 2 and 5 unique.
 */
INFLASH(eth_addr_t myMACset) = {{0x40, 0x1B, 0x50, 0xCA, 0xFE, 0x00}};

/* The device's basic default configuration data
 *
 *  A structure of this type holds basic configuration data specific to the
 *  individual device. To survive power off one copy of these data is to be
 *  held in EEPROM at address
 *  \ref eeConfigAdd = EEPROM[ \ref EEPROM_POINTER2_EE_CONF ].
 */
conf_data_t defaultConfData;

/* The device type's basic default configuration data in flash memory
 *
 *  A structure of this type holds basic configuration data specific to the
 *  device type. These values typically are used only as long as no other
 *  device specific (individual) values were set in persistent EEPROM storage.
 */
INFLASH (conf_data_t defaultTypeConfData) = {
      {{0x40, 0x1B, 0x50, 0xCA, 0xFE, 0x00}}, // MAC

      0xA1,  // magic number ;  til 07.02.2012 0xA0

      /* The default IP configuration */
      {IP_ADD(192, 168, 89, 147), // IP address
       IP_ADD(255, 255, 255,  0), // net mask
       IP_ADD(192, 168, 89,  11), // default router
       IP_ADD(192, 168, 89,  11), // DNS
       IP_ADD(0, 0, 0,  0), // DNS 2
       IP_ADD(192, 168, 89,  2), // NTP
       IP_ADD(192, 168, 89,  3), // NTP 2
       604800, // lease time (one week)
       0,        // set time (unused)
       DNS1_MSK, // consider only DNS set
       DNS1_MSK | DHCP_MSK  // use DHCP and NTP if possible
      }
};

/* Initialise persistence / EEPROM handling */
void persistInit(void){

   uint8_t freshBoard = 0;
    if ( eeReadBytes(EEPROM_POINTER2_EE_CONF, (uint8_t*)&eeConfigAdd, 2)) {
       if (eeConfigAdd < 64 || eeConfigAdd > (EEP_SIZE - 128)) { // no add
          eeConfigAdd = DEFAULT_EEPROM_CONF_ADD;

          // Attn.: spin waiting -- for init phases only
          while (!eeWriteByte(EEPROM_POINTER2_EE_CONF, LBYTE(eeConfigAdd)));
          while (!eeWriteByte(EEPROM_POINTER2_EE_CONF+1, HBYTE(eeConfigAdd)));
          while ((EECR & (1<<EEPE)) ||  (SPMCSR & (1<<RWWSB)) ); // wait end

         // eeConfigAdd = 0; // no valid EEPROM configuration data
          freshBoard = 0x10;  // not valid config add replaced
       } // no add
    } else {
       eeConfigAdd = 0; // no valid configuration data
       freshBoard = 2;  // not readable
    }

    if (eeConfigAdd) {
       eeReadBytes(eeConfigAdd, (uint8_t*)&defaultConfData,
                                                        sizeof (conf_data_t));
       if (defaultConfData.magicNumber
             != pgm_read_byte(&(defaultTypeConfData.magicNumber))) {
          freshBoard |= 4;  // last write failed or struct type changed
          eeConfigAdd = 0;
       }
    }

    if (!eeConfigAdd) {
       freshBoard |= 0x20;
       eeConfigAdd = DEFAULT_EEPROM_CONF_ADD;

       while (!eeWriteByte(EEPROM_POINTER2_EE_CONF, LBYTE(eeConfigAdd)));
       while (!eeWriteByte(EEPROM_POINTER2_EE_CONF+1, HBYTE(eeConfigAdd)));
       memcpy_P(&defaultConfData, &defaultTypeConfData, sizeof(conf_data_t));

       if (!initEEthreadWrite(&eeOpData,  eeConfigAdd,
                         (uint8_t*)&defaultConfData, sizeof(conf_data_t))) {
          freshBoard |= 0x40;
       } else { // (miss-) use thread function as spin wait
          while (eeOperationThread(&eeOpData) < PT_EXITED);
       }
    }
    if (freshBoard) {
       bufLogTxt_P(txtFreshBoard);
       bufLogHex(freshBoard);
       bufLogLF(2);
    }

    setMACadd(&defaultConfData.macAddress);  // set default MAC address
} // persistInit(void)
