/**
 *  \file proc_io.c
 *
 *  weAutSys'/weAut_01's (low level) system calls and services
 *           for process I/O and HMI
 *
 *  This file contains the implementation of \ref intro_sec "weAutSys" /
 *  weAut_01  (low level) system calls and services for process input and
 *  output (I/O) and I/O for the human machine interface (HMI). This is
 *  system software and must not be modified for user or application programs.
 *
 * This file is part of weAutSys &nbsp;
 *           <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2011 Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp; <<a href="http://a-weinert.de">a-weinert.de</a>>
 * 2
 * 25.01.2017
 */
//  SVN last change by A. Weinert

#include "we-aut_sys/common.h"
#include "we-aut_sys/spi.h"
#include "we-aut_sys/proc_io.h"
#include "we-aut_sys/ll_system.h"
#include "we-aut_sys/system.h"
#include <avr/io.h>

// the next two includes are rubbish for Eclipse Indigo .h troubles only
//#include <avr/io.h>
//#undef _AVR_IOXXX_H_
//#  include <avr/iom1284p.h>
// delete three # above as soon as Eclipse can handle nested .h files again


static uint8_t  prvDiL, actDiL;

/* Turn DI display LEDs off */
void offDiLEDs(){
   if (actDiL) toDiLEDs(0);
} //  offDiLEDs()

/* Output to DI display LEDs.
 *
 * 2s with clock / 4 * 1
 */
void  toDiLEDs(uint8_t value){
   if (value == actDiL) return; // nothing to do
#if defined(weAut_01) || defined(weAut_00)
   // comment in for SPI out one byte measurement
   // setTestPin0(ON);   // Signal start output (reset by int (as \CS))
   selectDIdisplayLEDs();
   SPDR = value;         // BTW start SPI and do something useful
   prvDiL = actDiL;
   actDiL = value;
   andBP( ~(1<<2) ); // turn them on
   deSelectSPI1();
   //  setTestPin0(OFF); // measure no int time interrupt
#else
   prvDiL = actDiL;
   actDiL = value;
#if defined(DI_LEDs0_PORT)
   toPort(DI_LEDs0_PORT,value);
#endif // DI_LEDs0_PORT
#endif
} //  toDiLEDs(uint8_t)

/* Turn DI display LEDs on */
void onDiLEDs(){
   if (actDiL || !prvDiL) return; // is on or would stay off
   toDiLEDs(prvDiL);
} // onDiLEDs()

/* DI display LEDs: previous value */
uint8_t prvDiLEDs(){  return prvDiL;  }

/* DI display LEDs: actual value */
uint8_t actDiLEDs(){  return actDiL; }


//--------------------------------------------------

static uint8_t actDO, prvDO, enabDO;

/* Output to digital (process) output DO  */
void toDOdriver(uint8_t value){
   if (value == actDO) return;
#if defined(weAut_01) || defined(weAut_00)
   selectDOdriver();
   SPDR = value;
   prvDO = actDO;
   actDO = value;
   deSelectSPI1();
#else
   prvDO = actDO;
   actDO = value;
   if (!enabDO) return;
#if defined(DO_0_PORT)
   toPort(DO_0_PORT,value);
#endif //  DO_0_PORT
#endif // not weAut_0x
} // toDOdriver()

/* DO digital (process) output: actual value  */
uint8_t actDOdriver(void){  return actDO; }

/* DO digital (process) output: previous value  */
uint8_t prvDOdriver(void){  return prvDO; }


/* Enable the digital (process) output DO driver  */
void enableDOdriver(uint8_t state){
   if (state) state = ON;
   if (state == enabDO) return; // do nothing on no change
   enabDO = state;

#if defined(weAut_01) || defined(weAut_00)
   // Port C C5 enable DO driver
   if (enabDO) {
      orCP( (1<<5));
   } else {
     andCP(~(1<<5));
   }
#elif defined(DO_0_PORT)
   if (state) state = actDO;
   toPort(DO_0_PORT, state);
#endif // not weAut_0x && DO_PORT
} // enableDOdriver(uint8_t)

/* Digital (process) output DO driver enabled */
uint8_t doDriverEnabled(){ return enabDO; }

/* Digital (process) output DO driver OK  */
uint8_t doDriverOK(){
#if defined(weAut_01) || defined(weAut_00)
   // Port C C4 DO driver /fault
   return PINC & (1<<4);
#elif defined(DO_0_PORT)
   return inPort(DO_0_PORT) == (enabDO & actDO);
#else
   return ON; // true = OK
#endif
} // doDriverOK

//------------------------------------------------------------


/* Digital (process) input DI (direct) */
uint8_t dctDI(){
#if defined(weAut_01) || defined(weAut_00) || defined(arduMega2560)
   return PINA;  // 8 DIs on weAut_01 are all Port A
#else
   return 0;
#endif
} // dctDI()

/** internal helper: pull up resistors switch */
void pullDIup(uint8_t msk)  __attribute__((always_inline));
void pullDIup(uint8_t msk)  {
#if defined(weAut_01) || defined(weAut_00) || defined(arduMega2560)
   PORTA = msk;  //  8 DIs on weAut_01 are all Port A
#endif
} // pullDIup

/* Shift DI thresholds up mask permanently */
uint8_t upDIthreshForce;

/* Shift DI thresholds up mask conditionally / larger hysteresis  */
uint8_t upDIthresh4hyst;

uint8_t diAct, diPrv, diLbp, diFil, diLFi;

/* Actual digital (process) input DI    */
uint8_t actDI(){ return diAct;   }

/* Previous digital (process) input DI  */
uint8_t prvDI(){  return diPrv;  }

/* Last before previous digital (process) input DI    */
uint8_t lbpDI(){  return  diLbp; }

/* The final or filtered digital (process) input DI   */
uint8_t filDI(){  return diFil;  }


//--------- DI / AI  ---------------

/* Use channel(s) as AI instead of DI */
uint8_t aiChannels;

/* Analogue input available */
uint8_t aiConvd;

/* Analogue input results */
uint8_t aiResult[8];

// 0xE0 :  2,56V , left adjust (8 bit resol)
// 0x60 :  VCC   , left adjust (8 bit resol)
// 0xA0 :  1,1V  , left adjust (8 bit resol)
static uint8_t muxMode =  0x60;

static uint8_t actADCchannel; // 0..7 set by interrupt and init
static uint8_t actADCchanMsk; // sliding 1 consistent to actADCchannel


/* Set the usage of channel(s) as AI instead of DI */
void setAIchannels(uint8_t mask){
   if (mask == aiChannels) return;
   if (aiChannels) { // ADC was on
     if (!mask) { // shall be off
        ADCSRA = 0x07; // turn off
     } // shall be off
   } else { // ADC was off
      aiConvd = actADCchannel = 0; // 0..7 set by interrupt and init
      actADCchanMsk = 1; // sliding 1 consistent to actADCchannel
      while (!(actADCchanMsk & mask)) { // search first channel ordered
         ++actADCchannel;
         actADCchanMsk <<= 1;
      } // search first channel (there is at least one)
      // select modes and first channel
      ADMUX = muxMode + actADCchannel; // 2.56V , left adjust (8 bit resol)
      ADCSRA = 0xCF;  // turn it on, start conversion, enable interrupt
   }
   DIDR0 = aiChannels = mask;
} // setAIchannels(uint8_t)


/** Start conversion on first AD channel set (internal helper) */
void startFirstAIconv(void){
   if (!aiChannels) return;
   // setTestPin0(ON); // TEST: measure total ADC time

   actADCchannel = 0; // 0..7 set by interrupt and init
   actADCchanMsk = 1; // sliding 1 consistent to actADCchannel
   while (!(actADCchanMsk & aiChannels)) { // search first channel ordered
      ++actADCchannel;
      actADCchanMsk <<= 1;
   } // search first channel (there is at least one)
   // select modes and first channel
   ADMUX = muxMode + actADCchannel; // 2.56V , left adjust (8 bit resol)
   ADCSRA = 0xCF;  // leave it on, start conversion, enable interrupt
   //  setTestPin1(ON); // TEST: measure total ADC conversion time
} // startFirstAIconv()


/** The ADC complete interrupt */
ISR( ADC_vect ){
   // setTestPin1(OFF); // TEST: measure total ADC conversion time

   aiResult[actADCchannel] = ADCH; // just 8 bit resolution (left adjusted)
   aiConvd |= actADCchanMsk;
   do {
     ++actADCchannel;
     actADCchanMsk <<= 1;
     if(actADCchanMsk & aiChannels) { // next channel found
        ADMUX = muxMode + actADCchannel; // 2.56V , left adjust (8 bit resol)
        ADCSRA = 0xCF;  // leave it on, start conversion, enable interrupt
      //  setTestPin1(ON); // TEST: measure total ADC conversion time
        break;
     } else {      // next channel found
        // setTestPin0(OFF); // TEST: measure total ADC time
     } // no next channel
   } while (actADCchanMsk); // 1 shifted out
} //  ADC complete interrupt


//--------  PIN B states   ---------
uint8_t pbAct, pbPrv, pbLbp, pbFil;

/*  Load voltage low filtered (0..128)  */
uint8_t  lowLV;


/* Digital (process) input DI (system implementation)
 *
 * To be called in system ms thread (normally).
 */
void procDIcyc(){
   if (aiChannels) startFirstAIconv();    // start AD

   // filter enter key
   pbLbp = pbPrv;
   pbPrv = pbAct;
   pbAct = PINB;
   ///  setTestPin0(pbAct & 1); // Test : enter
   //       theOr  = pbAct | pbLbp | pbPrv;
   //              theAnd = pbAct & pbLbp & pbPrv;
   pbFil = ( pbFil & (pbAct | pbLbp | pbPrv) )
                         | (pbAct & pbLbp & pbPrv);

   // filter digital inputs
   diLbp = diPrv;
   diPrv = diAct;
   diAct = dctDI();

   diLFi = diFil;
   // 3 ms filtering    (for 2 ms omit diLbp)
   uint8_t theOr  = diAct | diLbp | diPrv;
   uint8_t theAnd = diAct & diLbp & diPrv;
   diFil = ( diFil & theOr ) | theAnd;
   if (diLFi != diFil) {  // a change
      pullDIup( (diFil & upDIthresh4hyst) | upDIthreshForce );
      // if (displayDIinLEDs)   /// make user configurable later
      toDiLEDs(diFil);
   } // a DI change

   /// setStatusLedRd(ACSR & 32);  // Test: Comparator (LV) direct to red LED
   if (ACSR & 32) { // LV is low
      if (!lowLV) {
         lowLV = 4; // keep on for at least 4 ms
         setStatusLedRd(ON);
      } else if (! (lowLV & 128)) {
         ++lowLV;
         setStatusLedRd(ON);
      }
   } else if (lowLV) { // LV is not low (but was)
      if (--lowLV == 0) setStatusLedRd(OFF);
   }  // LV is not low (but was)

} // procDIcyc()


//------------------------------------------------------------


/* Low level reset type initialisation of process I/O  */
void initProcIO(){

#if defined(weAut_00) || defined(weAut_01)
   // init SPI 1 : used for DI_LEDS and DO
   // Enable   SPI,         Master,      Int   (SPR1,0=00 : clock /4)
   SPCR = (1 << SPE)  | (1 << MSTR)     // no Int (1 << SPIE)
          | ( 1 << CPHA)  | ( 1 << CPOL);   // CPHA=CPOL=1 for 74HCT595
   SPSR = (1 << SPI2X);               // SPI2X  = ( clock / 4 ) * 2
   // SPI 1 runs with 1/2 CPU clock, ie. 10MHz, needing 16 clocks for one byte

   // init SPI 2 : used for memory card and Ethernet ENC28J60
   UBRR1 = 0;  // 16bit register; Baud 0 for init
   //  UCSR1A |= (1<<U2X1);  // Set U2X mode (the manual is unclear here)
   /* Set MSPI mode of operation and SPI data mode 0. */
   UCSR1C = (1<<UMSEL11) | (1<<UMSEL10) | (0<<UCPHA1) | (0<<UCPOL1);
   /* Enable receiver and transmitter. */
   UCSR1B = (1<<RXEN1)|(1<<TXEN1);
   /* Set baud rate  IMPORTANT: has to be set after above enable  */
   UBRR1 = spi2EtherChipBaud; // 16bit register;
   // 9 : 1MHz ; 1: 10 MHz  4: 2 MHz clock

   // init LEDs and DO

   selectDOdrivDIleds();
   SPDR = 0;  // send  0 to all selected SPI 1 devices;
   prvDiL = actDiL = 0;
   actDO =  prvDO = enabDO = 0;
   upDIthreshForce = upDIthresh4hyst = 0;
   diAct =  diPrv = diLbp =  diFil =  diLFi = 0;

   pbAct =  pbPrv =  pbLbp = pbFil = 0;
   deSelectSPI1();

#elif defined(arduMega2560)

   prvDiL = actDiL = 0;
   actDO =  prvDO = enabDO = 0;
   upDIthreshForce = upDIthresh4hyst = 0;
   diAct =  diPrv = diLbp =  diFil =  diLFi = 0;

   pbAct =  pbPrv =  pbLbp = pbFil = 0;

#elif defined(arduinoUno)
   // todo
#endif

   // init analogue comparator (AC)
   ADCSRB = 0;       // ACME = 0 : switch on AC
   ACSR   = (1 << ACBG);  // AC: Bandgap as AC's "+input"
   DIDR1  = 2;     // disable AIN1 = PB3 = "-input" as digital input
   lowLV  = 5;    // turn off red LED after 5 ms stable LV

   aiChannels = aiConvd = 0;
} // initProcIO()

