// Copyright (c) 2012 Albrecht Weinert, Bochum
// $Id: smc.c 2 2017-01-25 16:49:03Z albrecht $

/**
 * \file smc.c
 *
 *  weAutSys' (low level) system calls, services and types
 *           for communication with a small memory card
 *
 *  This file contains some implementations for weAutSys' (low level) system
 *  calls, services and types to communicate via SPI with a
 *  \ref memCard_sec "small memory card".
 *
 *  This is system software and must not be modified for user or
 *  application programs.
 *
 * This file is part of weAutSys &nbsp;
 *           <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 *      <br />
 * This source is for weAut_00 and weAut_01 only. Other platforms
 * (arduMega2560, arduinoUno) must not include/translate it respectively have
 * a specific variant.
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2011 Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp; <<a href="http://a-weinert.de">a-weinert.de</a>>
 * $Revision: 2 $
 * $Date: 2017-01-25 16:49:03 +0000 (Wed, 25 Jan 2017) $
 */
//  SVN last change by $Author: albrecht $

#include "we-aut_sys/smc.h"
#include "we-aut_sys/common.h"
#include "we-aut_sys/ll_system.h"

/*  The (one / first) small memory card (SMC) state  */
struct smcThr_data_t smcState; // the SMC (thread) state


#include "we-aut_sys/spi.h"
#include "we-aut_sys/utils.h"
#include "we-aut_sys/smc2fs.h"


#include <avr/pgmspace.h>
#include <string.h>
#include "pt/pt.h"  // for sake of Eclipese only

// the next two includes are rubbish for Eclipse Indigo .h troubles only
//#include <avr/io.h>
//#undef _AVR_IOXXX_H_
//#  include <avr/iom1284p.h>
// delete three # above as soon as Eclipse can handle nested .h files again

#if DEBUG_SMC
#  include "we-aut_sys/log_streams.h"
INFLASH(char smc80clksMes[]) = "\n  SMC 80clks ";
//                              0 123456789.123456789v.
INFLASH(char smcSetFrqMes[]) = "\n  SMC setFrq ";
INFLASH(char smcGoIdleMes[]) = "\n  SMC goIdle ";
INFLASH(char smcRdOCRMes[])  = "\n  SMC rd OCR ";
INFLASH(char smcGeTypeMes[]) = "\n  SMC geType ";
INFLASH(char smcRdCSDMes[])  = "\n  SMC rd CSD ";
INFLASH(char smcRdCIDMes[])  = "\n  SMC rd CID ";
#endif

/*  Hardware card detect
 *
 *  \return true: switch closed = card inserted
 */
uint8_t smcInsertSwitch(){
#if   SMC_INSERT == 0xD5
   return PIND & 0x20 ? 0 : 'i';
#else
   return 'm';
#endif
} // smcInsertSwitch()

/*  Have 80 dummy clocks */
void clk80(void){
   deSelectSPI2();
   spi2TranceiveN(0xFF, 10); // 80 dummy clocks with CS=H
   const uint8_t brr = UBRR1L;
   if (brr == SPI400kHz) {
      smcState.flag &= ~ CT_POWERD_UP; // clear power up action
      smcState.cardType |= CT_POWERD_UP; // set state
   }

#if DEBUG_SMC >= 1
   bufLogTxt_P(smc80clksMes); bufLogDecB(brr);
   bufLogPut2C(' ', '\n');
#endif
} // clk80()

/*  Set the SPI clock frequency for the memory card
 *
 *  \param frqUse control value to set SPI 2 clock for the memory card
 */
void smcSetFrq(const uint8_t frqUse){
#if DEBUG_SMC >= 1
   if (spi2MemCardBaud == frqUse) return; // no report on unchanged
   bufLogTxt_P(smcSetFrqMes); bufLogDecB(frqUse);
   bufLogPut2C(' ', '\n');
#endif
   spi2MemCardBaud = frqUse;
} // smcSetFrq(const uint8_t


/*  CRC7 look up table
 *
 *  For byte wise CRC calculations look up algorithms a an order of magnitude
 *  faster than bitwise shifting algorithms that mimic the shift register
 *  hardware implementation. This look up table is in program / flash memory.
 */
INFLASH(uint8_t crc7lookUp[]) = {
      0x00,  0x09,  0x12,  0x1B,  0x24,  0x2D,  0x36,  0x3F,
      0x48,  0x41,  0x5A,  0x53,  0x6C,  0x65,  0x7E,  0x77,
      0x19,  0x10,  0x0B,  0x02,  0x3D,  0x34,  0x2F,  0x26,
      0x51,  0x58,  0x43,  0x4A,  0x75,  0x7C,  0x67,  0x6E,
      0x32,  0x3B,  0x20,  0x29,  0x16,  0x1F,  0x04,  0x0D,
      0x7A,  0x73,  0x68,  0x61,  0x5E,  0x57,  0x4C,  0x45,
      0x2B,  0x22,  0x39,  0x30,  0x0F,  0x06,  0x1D,  0x14,
      0x63,  0x6A,  0x71,  0x78,  0x47,  0x4E,  0x55,  0x5C,
      0x64,  0x6D,  0x76,  0x7F,  0x40,  0x49,  0x52,  0x5B,
      0x2C,  0x25,  0x3E,  0x37,  0x08,  0x01,  0x1A,  0x13,
      0x7D,  0x74,  0x6F,  0x66,  0x59,  0x50,  0x4B,  0x42,
      0x35,  0x3C,  0x27,  0x2E,  0x11,  0x18,  0x03,  0x0A,
      0x56,  0x5F,  0x44,  0x4D,  0x72,  0x7B,  0x60,  0x69,
      0x1E,  0x17,  0x0C,  0x05,  0x3A,  0x33,  0x28,  0x21,
      0x4F,  0x46,  0x5D,  0x54,  0x6B,  0x62,  0x79,  0x70,
      0x07,  0x0E,  0x15,  0x1C,  0x23,  0x2A,  0x31,  0x38,
      0x41,  0x48,  0x53,  0x5A,  0x65,  0x6C,  0x77,  0x7E,
      0x09,  0x00,  0x1B,  0x12,  0x2D,  0x24,  0x3F,  0x36,
      0x58,  0x51,  0x4A,  0x43,  0x7C,  0x75,  0x6E,  0x67,
      0x10,  0x19,  0x02,  0x0B,  0x34,  0x3D,  0x26,  0x2F,
      0x73,  0x7A,  0x61,  0x68,  0x57,  0x5E,  0x45,  0x4C,
      0x3B,  0x32,  0x29,  0x20,  0x1F,  0x16,  0x0D,  0x04,
      0x6A,  0x63,  0x78,  0x71,  0x4E,  0x47,  0x5C,  0x55,
      0x22,  0x2B,  0x30,  0x39,  0x06,  0x0F,  0x14,  0x1D,
      0x25,  0x2C,  0x37,  0x3E,  0x01,  0x08,  0x13,  0x1A,
      0x6D,  0x64,  0x7F,  0x76,  0x49,  0x40,  0x5B,  0x52,
      0x3C,  0x35,  0x2E,  0x27,  0x18,  0x11,  0x0A,  0x03,
      0x74,  0x7D,  0x66,  0x6F,  0x50,  0x59,  0x42,  0x4B,
      0x17,  0x1E,  0x05,  0x0C,  0x33,  0x3A,  0x21,  0x28,
      0x5F,  0x56,  0x4D,  0x44,  0x7B,  0x72,  0x69,  0x60,
      0x0E,  0x07,  0x1C,  0x15,  0x2A,  0x23,  0x38,  0x31,
      0x46,  0x4F,  0x54,  0x5D,  0x62,  0x6B,  0x70,  0x79  };

/*  Calculate CRC7 (one step)
 *
 *  \internal This is the (much faster) lookup variant en lieu de
   for (uint8_t i = 8; i; --i) {
      crc <<= 1;
      if ((datByte ^ crc) & 0x80)
         crc ^= 0x09;
      datByte <<= 1;
   }
   return crc;
 *
 *  \param crcIn the crc before; use 0 in first setp
 *  \param datByte the next 8 bits for the CRC
 *  \return the new CRC
 */
uint8_t crc7stp(uint8_t crc, const uint8_t datByte){
   crc = (crc << 1) ^ datByte;
   return pgm_read_byte(crc7lookUp + crc);
} // crc7stp(2 * uint8_t)


/*  De-select the small memory card
 *
 *  This (chip-) de-selects the SMC. It has to be used when all
 *  \ref sendCmd "commands" and write / reads are done.
 */
void deSelectSMC(void){ deSelectSPI2(); }


/* Send a command to the small memory card
 *
 *  This function send the command, the arguments and the \ref crc7stp "CRC7"
 *  to the memory card. Returned is the card's answer if any. Several attempts
 *  are made to get a correct answer determined by bit 7 zero. If the answer
 *  is 0xFF the card may be gone.
 *
 * \param cmdNum   command number
 * \param cmdArg   the 4 byte (32 bits) command arguments
 */
uint8_t sendCmd(uint8_t cmdNum, uint32_t cmdArg){
    // to front to see busy earlier (08.11.12)
   cmdNum &= 0x3F;
#if DEBUG_SMC >= 2
   //           0123456789.123456789v123456789t12 3
   char tS[] = " ## SMC CMD..(o......o) .c> rr  \n";
   twoDigs(&tS[11], cmdNum);
   eightHexs(&tS[14], cmdArg);
#endif
   cmdNum |= 0x40;
   selectMemCard();  // select is here again here (w/o clock: no effect)
   spi2Transmit(0xFF);
   spi2Transmit(cmdNum);
   uint8_t crc = crc7stp(0, cmdNum);
   spi2Receive();  // should not wait long

   cmdNum =  (uint8_t)(cmdArg >> 24);       // cmdNum used for data now
   spi2Transmit(cmdNum); crc = crc7stp(crc, cmdNum);
   spi2Receive();  // should not wait long

   cmdNum =  (uint8_t)(cmdArg >> 16);
   spi2Transmit(cmdNum); crc = crc7stp(crc, cmdNum);
   spi2Receive();  // should not wait long

   cmdNum =  (uint8_t)(cmdArg >> 8);
   spi2Transmit(cmdNum); crc = crc7stp(crc, cmdNum);
   spi2Receive();  // should not wait long

   cmdNum =  (uint8_t)cmdArg;
   spi2Transmit(cmdNum); crc = crc7stp(crc, cmdNum);

// this is the copy and paste block EOC start
#if DEBUG_SMC >= 2 & 0
   twoHexs(&tS[6], crc);  /// TEST only get crc
#endif
   uint8_t chkPlug = spi2Receive();  // should not wait long

   spi2Transmit((crc<<1) | 1);
   chkPlug &= spi2Receive();  // should not wait long

   spi2Transmit(0xFF); // 8 clocks needed anyway
   chkPlug &= spi2Receive();  // should not wait long (paired with CRC)

   chkPlug &= spi2Receive();  // last receive paired with FF; may wait a bit

   crc = NCR_EXTR_WAIT; // try .. receives for answer max.
   do {
      chkPlug &= cmdNum = spi2Tranceive(0xFF);
   } while ((cmdNum & 0x80) && --crc);

   if (!crc && chkPlug == 0xFF) {
      smcState.cardType &= ~CT_INS_POWD;
   }
// this is the copy and paste block EOC end

#if DEBUG_SMC >= 2
   tS[24] = ('0' + NCR_EXTR_WAIT) -crc;
   twoHexs(&tS[28], cmdNum);
   if (spi2Errors) {
      twoHexs(&tS[6], spi2Errors);
      spi2Errors = 0;
   }
   bufLogPutSt(tS, &bufLogStreams);
#endif
   return SMC_LAST_R1 = cmdNum;
} // sendCmd(uint8_t, uint32_t)

/* Send a command with 0 argument to the small memory card
 *
 *  This function acts as <br /> &nbsp;  &nbsp;  &nbsp;
 *     \ref sendCmd "sendCmd(cmdNr, 0)";   <br />
 *  just to improve this frequent case.
 * \param cmdNr    command number (is ORed with 0x40, the pure command number
 *                 is sufficient)
 */
uint8_t sendCmd0arg(uint8_t cmdNum){
   cmdNum &= 0x3F;
#if DEBUG_SMC >= 2
   //           0123456789.123456789v123456789t1234 5
   char tS[] = " ## SMC CMD..(0x00)     .c> rr \n";
   twoDigs(&tS[11], cmdNum);
#endif

   cmdNum |= 0x40;
   selectMemCard();
   spi2Transmit(0xFF);
   spi2Transmit(cmdNum);
   uint8_t crc = crc7stp(0, cmdNum);
   spi2Receive();  // should not wait long

   spi2Transmit(0); crc = crc7stp(crc, 0);
   spi2Receive();  // should not wait long

   spi2Transmit(0); crc = crc7stp(crc, 0);
   spi2Receive();  // should not wait long

   spi2Transmit(0); crc = crc7stp(crc, 0);
   spi2Receive();  // should not wait long

   spi2Transmit(0); crc = crc7stp(crc, 0);

   // this is the copy and paste block EOC start
   #if DEBUG_SMC >= 2 & 0
      twoHexs(&tS[6], crc);  /// TEST only get crc
   #endif
      uint8_t chkPlug = spi2Receive();  // should not wait long

      spi2Transmit((crc<<1) | 1);
      chkPlug &= spi2Receive();  // should not wait long

      spi2Transmit(0xFF); // 8 clocks needed anyway
      chkPlug &= spi2Receive();  // should not wait long (paired with CRC)

      chkPlug &= spi2Receive();  // last receive paired with FF; may wait a bit

      crc = NCR_EXTR_WAIT; // try .. receives for answer max.
      do {
         chkPlug &= cmdNum = spi2Tranceive(0xFF);
      } while ((cmdNum & 0x80) && --crc);

      if (!crc && chkPlug == 0xFF) {
         smcState.cardType &= ~CT_INS_POWD;
      }
   // this is the copy and paste block EOC end

#if DEBUG_SMC >= 2
   tS[24] = ('0' + NCR_EXTR_WAIT) -crc;
   twoHexs(&tS[28], cmdNum);
   if (spi2Errors) {
      twoHexs(&tS[6], spi2Errors);
      spi2Errors = 0;
   }
   bufLogPutSt(tS, &bufLogStreams);
#endif
   return SMC_LAST_R1 = cmdNum;
} // sendCmd0arg(uint8_t)


/** Send an application specific command to the small memory card
 *
 *  This function acts as <br /> &nbsp; &nbsp; &nbsp;
 *     \ref sendCmd0arg "sendCmd0arg(APP_CMD)";  <br /> &nbsp; &nbsp; &nbsp;
 *     \ref sendCmd "sendCmd(cmdNr, cmdArg)";    <br />
 *  besides some slight improvements.
 *
 * \param appCmdNum   application specific command number
 * \param cmdArg   the 4 byte (32 bits) command arguments
 * \sa GO_IDLE_STATE
 */
uint8_t sendAppCmd(uint8_t appCmdNum, uint32_t cmdArg){
   appCmdNum &= 0x3F;
#if DEBUG_SMC >= 2
   //           0123456789.123456789v123456789t12 3
   char tS[] = " ## SMC APP..(o......o) .c> rr  \n";
   twoDigs(&tS[11], appCmdNum);
   eightHexs(&tS[14], cmdArg);
#endif
   // send the APP_CMD prefix
   selectMemCard();
   spi2Transmit(0xFF);
   spi2Transmit(APP_CMD | 0x40);
   spi2Receive();  // should not wait long

   spi2Transmit(0);
   spi2Receive();  // should not wait long

   spi2Transmit(0);
   spi2Receive();  // should not wait long

   spi2Transmit(0);
   spi2Receive();  // should not wait long

   spi2Transmit(0);
   uint8_t chkPlug = spi2Receive();  // should not wait long

   // this is almost the copy and paste block EOC start
#if DEBUG_SMC >= 2 & 0
   twoHexs(&tS[6], crc);  /// TEST only get crc
#endif

   spi2Transmit((0x32<<1) | 1);
   chkPlug &= spi2Receive();  // should not wait long

   spi2Transmit(0xFF); // 8 clocks needed anyway
   chkPlug &= spi2Receive();  // should not wait long (paired with CRC)

   chkPlug &= spi2Receive();  // last receive paired with FF; may wait a bit

   uint8_t crc = NCR_EXTR_WAIT; // try .. receives for answer
   uint8_t appCmdPrfResp;
   do {
      chkPlug &= appCmdPrfResp = spi2Tranceive(0xFF);
   } while ((appCmdPrfResp & 0x80) && --crc);

   if (!crc && chkPlug == 0xFF) {
      smcState.cardType &= ~CT_INS_POWD;
   }
   // this is almost the copy and paste block EOC end

   if (appCmdPrfResp > 1) { // AppCMD prefix failed
      appCmdNum = appCmdPrfResp ;
      crc = ('0' + NCR_EXTR_WAIT) - ':';
      goto itsOver;
   } // AppCMD prefix failed

   // send command
   appCmdNum |= 0x40;
   spi2Transmit(0xFF);
   spi2Transmit(appCmdNum);
   crc = crc7stp(0, appCmdNum);
   spi2Receive();  // should not wait long

   appCmdNum =  (uint8_t)(cmdArg >> 24);
   spi2Transmit(appCmdNum); crc = crc7stp(crc, appCmdNum);
   spi2Receive();  // should not wait long

   appCmdNum =  (uint8_t)(cmdArg >> 16);
   spi2Transmit(appCmdNum); crc = crc7stp(crc, appCmdNum);
   spi2Receive();  // should not wait long

   appCmdNum =  (uint8_t)(cmdArg >> 8);
   spi2Transmit(appCmdNum); crc = crc7stp(crc, appCmdNum);
   spi2Receive();  // should not wait long

   appCmdNum =  (uint8_t)cmdArg;
   spi2Transmit(appCmdNum); crc = crc7stp(crc, appCmdNum);
   spi2Receive();  // should not wait long

   spi2Transmit((crc<<1) | 1);
   spi2Receive();  // should not wait long

   spi2Transmit(0xFF); // 8 clocks needed anyway
   spi2Receive();  // should not wait long (paired with CRC)

   spi2Receive();  // last receive paired with FF; may wait a bit

   crc = NCR_EXTR_WAIT; // try .. receives for answer max.
   do {
      appCmdNum = spi2Tranceive(0xFF);
   } while ((appCmdNum & 0x80) && --crc);
   //  deSelectSPI2();
   itsOver:
#if DEBUG_SMC >= 2
   tS[24] = ('0' + NCR_EXTR_WAIT) -crc;
   twoHexs(&tS[28], appCmdNum);
   if (spi2Errors) {
      twoHexs(&tS[6], spi2Errors);
      spi2Errors = 0;
   }
   bufLogPutSt(tS, &bufLogStreams);
#endif
   return SMC_LAST_R1 = appCmdNum;
} // sendAppCmd(uint8_t, uint32_t)


/*  Put card to idle state
 *
 *  This function should run once a small memory card is inserted (or is
 *  believed to have been). Returned is the "R1" answer to the
 *  \ref GO_IDLE_STATE "soft reset" command. Only the value 1 is OK meaning
 *  card is present and (now) in idle state. All other values are faults.
 *  0xFF (no valid R1 response) usually means no card inserted or card not
 *  responding at all.
 *
 *  \param mode <br />
 *           Bit 0 set (1): force  \ref SPI400kHz "400 kHz" and restore
 *           \ref  spi2MemCardBaud to previous value afterwards <br />
 *           Bit 1 set (2): omit the 80 dummy clocks before the
 *            \ref GO_IDLE_STATE "go idle" command
 *           Bit 2 set (4): omit the \ref GO_IDLE_STATE "go idle" command
 *                          (0 is returned in that case)
 *  \return  answer (R1) for the \ref GO_IDLE_STATE "command" sent: 0x01 is
 *           OK (card in slot and in idle state)
 */
uint8_t smcSetIdle(uint8_t mode){
   smcState.cardType = 0;
   const uint8_t saveBaud = spi2MemCardBaud;
   if (mode & 1) smcSetFrq(SPI400kHz); // set to 400kHz
   if (!(mode & 2)) {
      selectMemCard();    // make effective and give short trigger
      smcSetFrq(saveBaud);
      clk80();          // 80 dummy clocks with CS and DI (MoSi) high
   }
   if (!(mode & 4)) {
      if (sendCmd0arg(GO_IDLE_STATE) == 1) { // set idle succ
         smcState.cardType |= CT_POWERD_UP | CT_INSERTED; // set state
         smcState.flag &= ~ CT_INSERTED; // clear action
      } else { // set idle succ / fail
         smcState.cardType = 0;
      }
      //  deSelectSMC(); // already by clk80
#if DEBUG_SMC >= 1
      bufLogTxt_P(smcGoIdleMes); bufLogHex(SMC_LAST_R1);
      bufLogPut2C(' ', '\n');
#endif
       return SMC_LAST_R1;
   }  // idle command
    // no go idle commad
   return 0;
} // smcSetIdle(uint8_t)

/*  Check if card is busy
 *
 *  This function returns 0 (false not busy) if the SMC is inserted and
 *  responds 0xFF at least twice to two dummy reads. The two consecutive
 *  not busy answers are looked for 3 .. (3 + \c tries) times.
 *
 *  If the outcome is not busy the card is not de-selected.
 *  \param tries 2..255 maximum number of dummy check reads
 *         (byte clock cycles) used
 *  \return 0: not busy; 0xFF: no card inserted and powered up; 1: busy
 */
uint8_t checkBusy(uint8_t tries){
   uint8_t ret = 0xFF;
#if DEBUG_SMC >= 2
   //           0123456789.123456789v12345 6
   char tS[] = " ## SMC chkBsy ..>.. nC  \n";
   twoHexs(&tS[15], tries);
#endif
   if (!(smcState.cardType & (CT_POWERD_UP | CT_INSERTED))) goto weGo; // card
   selectMemCard();
   spi2Transmit(0xFF);
   uint8_t b0 = 0;
   uint8_t b1 = 0;

   do {
      b1 = b0;
      spi2Transmit(0xFF);
      b0 = spi2Receive();  // should not wait long
   } while ((b1 & b0) != 0xFF && tries--);
   b1 = spi2Receive();  // last read
   if  ((b0 & b1) == 0xFF) {
      ret =  0; // not busy
#if DEBUG_SMC >= 2
      tS[21] = 'r';
#endif
      goto weGo;
   }
   deSelectSMC();
   ret = 1; // busy
#if DEBUG_SMC >= 2
   tS[21] = 'b';
#endif
   // the end
   weGo:
#if DEBUG_SMC >= 2
   twoHexs(&tS[18], tries);
   bufLogPutSt(tS, &bufLogStreams);
#endif
   return ret; // no card
} // checkBusy


//---------------------- end driver functions / start thread and state ------



/* The small memory card (smc) handling thread
 *
 *  If some basic handling, like initialising, has to be done with a SMC
 *  this protothread thread will do it reasonable steps, dividing long
 *  running procedures by yielding.
 */
ptfnct_t smcThreadF(){
   if (!smcInsertSwitch()) { // check card insert switch
      if (smcState.cardType){ // went off from any operable state
         const uint8_t newFlags = smcState.flag & 0x80 ? FL_INI
               : smcState.flag & FL_INI; // ? set : leave only init. actions
         initSMCthreadState(newFlags); // leave only init. actions
      } // went off from any operable state
      return smcState.retC = PT_ENDED;
   } // no card by switch (may be wrongly inserted, if switch not configured)

   if (!(smcState.cardType & CT_INSERTED)) { // no card by SW diag.
      if ((smcState.cardType & CT_TYPE_MASK)  // went off from fully operable
            || (smcState.flag & (FL_CAT | FL_RDS | FL_WRS))) { // or complex op
         const uint8_t newFlags = smcState.flag & 0x80 ? FL_INI
               : smcState.flag & FL_INI; // ? set : leave only init. actions
         initSMCthreadState(newFlags); // leave only init. actions
      }  // went off from fully operable state (i.e. card type was determined)
   } // no card by command response diagnostic

   uint8_t pendingActions = smcState.flag & FL_IMP; // msk 7: check & 0: not impl.

   if (!pendingActions) { // no pending actions
      PT_INIT(&smcState.pt); // reset threading loop to start
      if (smcState.flag & 0x80) { // only check flag
         if (!(smcState.cardType & CT_INSERTED)) pendingActions= CT_INSERTED;
         if (!(smcState.cardType & CT_POWERD_UP)) pendingActions |= CT_POWERD_UP;
         if (!(smcState.cardType & CT_TYPE_MASK)) pendingActions |= FL_CAT;
         if (pendingActions) { // now do something
            smcState.flag = pendingActions | 0x80; // (re-) store in flag
            if (othersAskPrio()) return smcState.retC = PT_YIELDED;
         } else {
            smcState.flag = 0x80; // have it clear from unimplemented actions
         }
      } else { // only check flag | nothing at all
         smcState.flag = 0; // have it clear from unimplemented actions
      }
      ///  SW action to detect card going
#if SMC_INS_POLL
      sendCmd0arg(SEND_STATUS);
      if(!smcState.lastCmdResp[0]) smcState.lastCmdResp[1] = spi2Receive();
      deSelectSMC();
#endif
      return smcState.retC = PT_EXITED; // exit
   } else if (pendingActions & (FL_RDS | FL_WRS)) { // read | write------------------
         if (!(smcState.cardType & CT_TYPE_MASK)) { // check precond
            pendingActions |= FL_CAT;
            smcState.flag |= FL_CAT;
         } // check precond
   } // read  or write


   // -- to here only with pending action(s)
   PT_BEGIN(&smcState.pt);
   for(;;) { // protothreading as loop (entered only on pending action)

      if (pendingActions & CT_POWERD_UP) {
         if (othersAskPrio()) PT_YIELD(&smcState.pt);
         // we do not check if the flag is still set (here)
         if (pendingActions & CT_INSERTED) {
            // remark: at least one 8GByte card needs this in one step
            smcSetIdle(1); // 80 clocks  and go idle with 400 kHz
         } else {
            smcSetIdle(5); // only 80 clocks with 400 kHz
         }
         PT_YIELD(&smcState.pt); // yield after
      } // CT_POWERD_UP

      if (pendingActions & CT_INSERTED) { // go idle
         if (!(smcState.cardType & CT_POWERD_UP)) { // check precond
            pendingActions |= CT_POWERD_UP;
            smcState.flag |=  CT_POWERD_UP;
            continue; // to start loop
         } // check precond
         if (othersAskPrio()) PT_YIELD(&smcState.pt);
         // we do not check if the flag is still set (here)
         smcSetIdle(3); // go idle with 400 kHz
         PT_YIELD(&smcState.pt); // yield after
      } // CT_POWERD_UP (go idle)


      if (pendingActions & FL_CAT) { // check card type and leave idle
         if (!(smcState.cardType & CT_INSERTED)) { // check precond
            pendingActions |=  CT_INS_POWD;
            smcState.flag |= CT_INS_POWD;
            continue; // to start loop
         } // check precond
         smcState.cardType &= CT_INS_POWD; // clear type and OCR bits
         smcState.bufferState = 0; // set nothing is known
         if (othersAskPrio()) PT_YIELD(&smcState.pt);
         // we do not check if the flag is still set (here)
         //  the card type determination process     --------------------
         if (sendCmd(SEND_IF_COND, 0x1AA) <= 1) { //  SD Version 2
            spi2ReceiveSN(smcState.lastCmdResp, 0, 4); // get last 4 bytes of R7 response
            deSelectSMC(); // end command SEND_IF_COND
            if (smcState.lastCmdResp[2] != 0x01 || smcState.lastCmdResp[3] != 0xAA) { // !card
               // action for unusable card (that is inserted) ??
               goto doneFail;
            } // card can't work at range 2.7-3.6V
            smcState.lastCmdResp[4] = 20;
            for(;;) {
               PT_YIELD(&smcState.pt); // yield after command
               sendAppCmd(SEND_OP_COND_APP, 0x40000000);
               deSelectSMC(); // end command
               if (!smcState.lastCmdResp[0]) break; // success
               if (!(smcState.cardType & CT_POWERD_UP) // no more responding
                     || !(--smcState.lastCmdResp[4])) goto doneFail; // || loop : fail
            } // loop SEND_OP_COND_APP

            smcReadOCR();
            if (smcState.lastCmdResp[0]) goto doneFail; // fail

            smcState.cardType |= ((smcState.ocr[0] & 0x40) ? CT_V_2 | CT_HC : CT_V_2) | (CT_POWERD_UP | CT_INSERTED);
            smcState.flag &= ~ FL_CAT; // clear action
            goto doneSucc; // success

         } else {  // SDv1 (or MMCv3)
            // internal: this branch hasn't been tested due to lack of cards
            deSelectSMC(); // end (failed) command SEND_IF_COND
            PT_YIELD(&smcState.pt); // yield after SEND_IF_COND, 0x1AA
            sendAppCmd(SEND_OP_COND_APP, 0); deSelectSMC();
            smcState.lastCmdResp[4] = 20; // max loop

            if (smcState.lastCmdResp[0] <= 1)    {  // SD Version 1
               for(;;) {
                  PT_YIELD(&smcState.pt); // yield after command
                  sendAppCmd(SEND_OP_COND_APP, 0); deSelectSMC();
                  if (!smcState.lastCmdResp[0]) break; // success
                  if (!(smcState.cardType & CT_POWERD_UP) // no more responding
                        || !(--smcState.lastCmdResp[4])) goto doneFail; // || loop : fail
               } // loop SEND_OP_COND_APP

               smcState.cardType |= CT_V_1 | (CT_POWERD_UP | CT_INSERTED);
               goto doneSucc; // success
            } else { // MMC Version 3
               for(;;) {
                  PT_YIELD(&smcState.pt); // yield after command
                  sendCmd0arg(SEND_OP_COND); deSelectSMC();
                  if (!smcState.lastCmdResp[0]) break; // success
                  if (!(smcState.cardType & CT_POWERD_UP) // no more responding
                        || !(--smcState.lastCmdResp[4])) goto doneFail; // || loop : fail
               } // loop SEND_OP_COND
               sendCmd(SET_BLOCKLEN, 512); deSelectSMC();
               if (smcState.lastCmdResp[0]) goto doneFail;
               smcReadOCR();
               if (smcState.lastCmdResp[0]) goto doneFail;
               smcState.cardType |= CT_V_3 | (CT_POWERD_UP | CT_INSERTED);
            } // MMC Version 3
         } // // SDv1 (or MMCv3)
         doneSucc:
         smcState.flag &= ~ FL_CAT; // clear action
         smcReadCSD();
         smcState.flag |= FL_RDS;  // order read block

         doneFail:
         smcState.rwSector = 0;    // block 0
         smcState.bufferState &= ~(FL_RDS | FL_WRS); // set out of sync
         fMountSMC();
#if DEBUG_SMC >= 1
         bufLogPutSt_P(smcGeTypeMes, &bufLogStreams);
         bufLogHex(smcState.cardType);
         bufLogLF(1);
#endif
         PT_YIELD(&smcState.pt); // yield after
      } // FL_CAT (check type and leave idle)   ------------------------

     if (pendingActions & FL_RDS) { // read sector  --------------------
        if (smcState.bufferState & FL_RDS) { // already done
           smcState.flag &= ~ FL_RDS; // clear action
        } else {
           if (othersAskPrio() || checkBusy(32)) {
              PT_YIELD(&smcState.pt); // yield
              continue;  // and re-check
           }
           readDataBlock(smcState.rwSector, smcState.sectorBuff);
           PT_YIELD(&smcState.pt); // yield after
        }
     } // FL_RDS) (read sector)                  ------------------------

     if (pendingActions & FL_WRS) { // write sector  --------------------
        if ((smcState.bufferState & (FL_RDS | FL_WRS) )
                              == FL_RDS) { // already done or still in sync
           smcState.flag &= ~ FL_WRS; // clear action
        } else {
           if (othersAskPrio()) PT_YIELD(&smcState.pt);
           writeDataBlock(smcState.rwSector, smcState.sectorBuff);
           PT_YIELD(&smcState.pt); // yield after
        }
     } // FL_WRS) (write sector)                  ------------------------


   } // protothreading as loop (entered only on pending action)
   PT_END(&smcState.pt)
} // smcThreadF(struct smcThr_data_t *)

/*  Initialise the  small memory card (smc) handling thread
 *
 *  This function resets all SMC state. It should be called on reset /
 *  restart and on SMC removal, failures and insertion.
 *
 *  \param smcThread pointer to the SMC (thread) state
 *  \sa smcState
 */
void initSMCthreadState(const uint8_t actionFlag){
   // init all to 0, NULL, false
   memset(&smcState, 0, sizeof(struct smcThr_data_t) - SMC_SEC_BUF_SZ);
   smcState.theSMCfun = smcThreadF;
   SMC_LAST_R1 = 0x80;
   smcState.flag = actionFlag;
} // initSMCthreadState(uint8_t)


/*  Read single data block
 *
 *  1 ms with 31 waits on data token
 *  1,5 ms with 181 waits (@ 5MHz)
 *  1,35 ms with  138
 *
 * \param  sector the sector number
 * \param  buff   pointer to where to write the received data
 *                (NULL not allowed)
 * \return 0: OK  or negative response of command \ref READ_SINGLE_BLOCK or
 *                or 0x80 if no data token came (time out; may try later)
 */
uint8_t readDataBlock(const uint32_t sector, uint8_t * buff){
   if (!(smcState.cardType & CT_TYPE_MASK)) return 0x80; // no type info
#if DEBUG_SMC >= 1
   //           0 123456789.123456789v123456789t1 2
   char tS[] = "\n  SMC readBl aaaaaaaa OK cNo  \n";
   eightHexs(&tS[14], sector);
#endif
   uint8_t cnt = 0;
   uint8_t ret = 0;

   // avoid repeat if we already got that
   if ((smcState.rwSector ==  sector)  && (smcState.bufferState & FL_RDS)) {
#if DEBUG_SMC >= 1
      tS[13] = '=';
#endif
      if(buff == smcState.sectorBuff) goto weGo2;  // just ready
      cnt = 0xFB;                                // or copy
      memcpy(buff, smcState.sectorBuff, 512);
      goto weGo2;
   } // already got it; we do not read again

   const uint32_t sn = smcState.cardType & CT_HC ? sector // hi cap
         : mul32withVal512(sector);
   ret = sendCmd(READ_SINGLE_BLOCK, sn);
   if (ret) goto weGo;

   while (spi2Tranceive(0xFF) != SMC_DAT_TOK) {
     if(++cnt > 252) {
        ret = 0x80;
        goto weGo; // give it up
     }
   }

   spi2ReceiveNS(buff, 512, 2);  // receive buffer, skip CRC

   if(buff == smcState.sectorBuff) { // successful read to own buffer
      smcState.rwSector = sector; // set number
      smcState.flag &= ~ FL_RDS; // clear action
      smcState.bufferState = (smcState.bufferState | FL_RDS) & ~ FL_WRS;
#if DEBUG_SMC >= 1
      tS[13] = '*';
#endif
   } // successful read to own buffer

   weGo:
   deSelectSMC();
   weGo2:
   SAMPLE_MIN_SP(1);
#if DEBUG_SMC >= 1
   if (ret) twoHexs(&tS[23], ret);
   if (cnt) twoHexs(&tS[27], cnt);
   bufLogPutSt(tS, &bufLogStreams);
#endif
   return ret;
} // readDataBlock(uint16_t sector, uint8_t * buff, uint16_t len)

/*  Write single data block
 *
 * \param  sector the sector number (LBA) as for \ref WRITE_BLOCK
 * \param  buff   pointer to where to get the send data from (NULL not allowed)
 * \return 0: OK  or negative response of command \ref WRITE_BLOCK or
 *                or 0x80 if no type information
 */
uint8_t writeDataBlock(const uint32_t sector, const uint8_t * buff){
   if (!(smcState.cardType & CT_TYPE_MASK)) return 0x80; // no type info
#if DEBUG_SMC >= 1
   //           0 123456789.123456789v1234567 89t1 2
   char tS[] = "\n  SMC writeB aaaaaaaa rr  \n";
   eightHexs(&tS[14], sector);
#endif
  // if (!(smcState.cardType & CT_HC)){ //low capacity
    //  sector = mul32withVal512(sector);
   //}
   const uint32_t sn = smcState.cardType & CT_HC ? sector // hi cap
                                                 : mul32withVal512(sector);

   const uint8_t ret = sendCmd(WRITE_BLOCK, sn);
   if (ret) goto weGo;

   spi2Transmit(0xFF); // send dummy byte
   spi2Tranceive(SMC_DAT_TOK);
   const uint8_t * buffLast = buff + (512 + 1);
   for(;;) {
      spi2Tranceive(*buff);
      if (++buff == buffLast) break;
   }
   spi2Tranceive(0x03); spi2Tranceive(0x30); // dummy CRC
   spi2Tranceive(0xFF); // receive paired with last CRC; send for data resp
   smcState.lastDatResp = spi2Receive(); // receive data response (and pair)

   if(buff == smcState.sectorBuff) { // successful write to own buffer
      smcState.rwSector = sector; // set number
      smcState.flag &= ~ FL_WRS; // clear action
      smcState.bufferState = (smcState.bufferState | FL_RDS) & ~ FL_WRS;
#if DEBUG_SMC >= 1
   tS[13] = '*';
#endif
   } // successful read to own buffer

   weGo:
   deSelectSMC();

#if DEBUG_SMC >= 1
   twoHexs(&tS[23], ret);
   bufLogPutSt(tS, &bufLogStreams);
#endif
   return ret;
} // writeDataBlock(uint16_t sector, uint8_t * buff, uint16_t len)

/* Order sector read (as background task)
 *
 *  This function sets \ref smcState "smcState.rwSector" by \c sector and
 *  initialises \ref smcState "smcState.sectorBuff" to be read from that
 *  sector.
 *
 * \param  sector the (new) sector number
 *                for \ref smcState "smcState.rwSector/.sectorBuff"
 * \return
 *   0: OK already done (\ref smcState "smcState.sectorBuff" in sync.), <br />
 *   1: will be done ASAP (in background task), call again later, <br />
 *   0x80: action not possible (no card, no type or other error)
 */
uint8_t doSectorRead(uint32_t sector){
#if DEBUG_SMC >= 1
   //           0 123456789.123456789v123456 789t1 2
   char tS[] = "\n  SMC doScRd aaaaaaaa rr \n";
   eightHexs(&tS[14], sector);
#endif
    uint8_t ret = 0;
   if (!(smcState.cardType & CT_TYPE_MASK)) { // no type info
      smcState.rwSector = sector;
      smcState.flag &= ~ FL_R_W; // clear action
      smcState.bufferState &= ~ FL_R_W; // clear state
      return 0x80;
   } // no type info

   if (sector == smcState.rwSector) { // same sector
      if ((smcState.bufferState & FL_R_W) == FL_RDS) goto offWeGo; // in sync
      goto orderAndGo; // order action
   } // same sector

   smcState.rwSector = sector;

   orderAndGo:
   ret = 1;
   smcState.bufferState &= ~ FL_R_W; // clear state
   smcState.flag |= FL_RDS; // order action

   offWeGo:
#if DEBUG_SMC >= 1
   twoHexs(&tS[23], ret);
   bufLogPutSt(tS, &bufLogStreams);
#endif
   return ret;
}  // doSectorRead(uint32_t)

/* Mark sector buffer data as modified (start)
 *
 *  This function sets \ref smcState "smcState.rwSector" by \c sector and
 *  marks as \ref smcState "smcState.sectorBuff" as modified (new) data for
 *  the respective sector. This neither performs any read or write operations
 *  nor orders such as background task.
 *
 * \param  sector the (new) sector number
 *                for \ref smcState "smcState.rwSector/.sectorBuff"
 */
void setSectorModified(uint32_t sector){
#if DEBUG_SMC >= 1
   //           0 123456789.123456789v123456 789t1 2
   char tS[] = "\n  SMC stScMd aaaaaaaa \n";
   eightHexs(&tS[14], sector);
   bufLogPutSt(tS, &bufLogStreams);
#endif
   smcState.rwSector = sector;
   smcState.flag &= ~ FL_R_W; // clear action
   smcState.bufferState = (smcState.bufferState & ~FL_RDS)
                                                 | FL_WRS; // set state
} // setSectorModified(uint32_t)

/* Order sector synchronisation (as background task)
 *
 *  This function checks if \ref smcState "smcState.sectorBuff" contains
 *  modified data (for sector \ref smcState "smcState.rwSector") and if so
 *  initialises \ref smcState "smcState.sectorBuff" to be written to that
 *  sector.
 *
 *  Example sector copy: \code
 *  PT_WAIT_ASYIELD_WHILE(&myThread, doSectorRead(sourceSect) == 1);
 *  if (doSectorSync()) {  optional error handling / exit  }
 *  setSectorModified(destinationSect);
 *  PT_WAIT_ASYIELD_WHILE(&myThread, doSectorSync() == 1);
 *  if (doSectorSync()) {  optional error handling / exit  }
 *  \endcode
 *
 * \return
 *   0: OK already done (\ref smcState "smcState.sectorBuff" in sync.), <br />
 *   1: will be done ASAP (in background task), call again later, <br />
 *   0x80: action not possible (no card, no type or other error)
 */
uint8_t doSectorSync(void){
#if DEBUG_SMC >= 1
   //           0 123456789.123456789v123456 789t1 2
   char tS[] = "\n  SMC doScSy aaaaaaaa rr \n";
   eightHexs(&tS[14], smcState.rwSector);
#endif
   uint8_t ret = 0;
   if (!(smcState.cardType & CT_TYPE_MASK)) { // no type info
      smcState.flag &= ~ FL_R_W; // clear action
      smcState.bufferState &= ~ FL_R_W; // clear state
      ret =0x80;
   } else {

      if ((smcState.bufferState & FL_R_W) != FL_RDS) { // in sync
         smcState.flag |= FL_WRS; // order action
         ret = 1;
      }
   }
#if DEBUG_SMC >= 1
   twoHexs(&tS[23], ret);
   bufLogPutSt(tS, &bufLogStreams);
#endif
   return ret;
} // doSectorSync()


//---------------------------------------------------------------------------

/*  Read the SMC's ORC
 *
 *  This function reads the operation condition register (OCR, 4 byte) of an
 *  inserted and powered up small memory card. In case of success 0 is
 *  returned and the result is stored in \ref smcState "smcState.ocr".
 *
 *  It is usually not necessary to call this function if \ref smcTypeD() is
 *  true as the OCR has been read during the card type determination process.
 *
 *  \return 0: OK or negative response of command \ref READ_OCR or
 *             or 0x80 if SCM isn't powered up.
 */
uint8_t smcReadOCR(){
   if (!(smcState.cardType & CT_POWERD_UP)) return 0x80;
   sendCmd0arg(READ_OCR);
   if (!smcState.lastCmdResp[0]) {
      SAMPLE_MIN_SP(2);
      spi2ReceiveSN(smcState.ocr, 0, 4); // get 4 bytes of R3 response (Hi-byte first)
      //  smcState.cardType |= CT_OCR_RD |  CT_INS_POWD;
#if DEBUG_SMC >= 1
      bufLogTxt_P(smcRdOCRMes); bufLog8HexBE(*((uint32_t*)smcState.ocr));
      bufLogPut2C(' ', '\n');
#endif
   }
   deSelectSMC();
   return smcState.lastCmdResp[0];
} // smcReadOCR()

/*  Read the SMC's CSD
 *
 *  This function reads the card specific data register (CSD, 16 byte) of an
 *  inserted and powered up small memory card. In case of success 0 is
 *  returned and the result is stored in \ref smcState "smcState.csd".
 *
 *  It is usually not necessary to call this function if \ref smcTypeD() is
 *  true as the OCR and CSD have been read during the card type
 *  determination process.
 *
 *  \return 0: OK  or negative response of command \ref SEND_CSD or
 *                or 0x80 if SCM isn't powered up.
 */
uint8_t smcReadCSD(){
   if (!(smcState.cardType & CT_POWERD_UP)) return 0x80;
   uint8_t ret = sendCmd0arg(SEND_CSD);
   if (ret) goto weGo;

   uint8_t cnt = 0;
   while (spi2Tranceive(0xFF) != SMC_DAT_TOK) {
     if(++cnt > 24) {
        ret = 0x80;
        goto weGo; // give it up
     }
   }
   spi2ReceiveNS(smcState.csd, 16, 2); // 16 bytes of response token, skip CRC
   //   spi2ReceiveSN(NULL, 2, 0);  // skip CRCreceive buffer

#if DEBUG_SMC >= 1
      bufLogTxt_P( smcRdCSDMes);
      bufLog8HexBE(*((uint32_t*)smcState.csd));
      bufLog8HexBE(*((uint32_t*)smcState.csd + 1));
      bufLog8HexBE(*((uint32_t*)smcState.csd + 2));
      bufLog8HexBE(*((uint32_t*)smcState.csd + 3));  bufLogPut2C(' ', '\n');
#endif
   weGo:
   deSelectSMC();
   return ret;
} // smcReadCSD()

/*  Get the sector count
 *
 *  If the card type is \ref smcTypeD() "determined" this function returns the
 *  number of (512 byte) sectors on the card and 0 otherwise.
 */
uint32_t smcGetSectCount(){
   ucnt32_t ret;
   ret.v32 = 0;
   if (!smcTypeD()) goto weGo;
   const uint8_t csdStruct = smcState.csd[0] & 0xC0;
   if (csdStruct == 0x40) { // 01b CSD Version 2.0
      ret.v8[1] = smcState.csd[9];
      ret.v8[2] = smcState.csd[8];
      ret.v8[3] = smcState.csd[7] & 0x3F; // all in all 22 bit
      ++ret.v32; // C_SIZE field (22 bit) + 1
      ret.v32 <<= 2; // * 2**10 cause of 512K as offset by one byte and <<2
   } else if (!csdStruct) { // 00b CSD Version 1.0
      uint8_t expo = (smcState.csd[5] & 0x0F)
                           + ((smcState.csd[9] & 3) << 1) - 7;
      if (smcState.csd[10] & 0x80) ++ expo;

      ret.v16[0] = (uint16_t)smcState.csd[7] << 2;
      ret.v8[0] += smcState.csd[8] >> 6;
      ret.v8[1] += (smcState.csd[6] & 3) << 2;
      ++ ret.v16[0];
      if (expo >= 8) {
         ret.v8[3] =  ret.v8[2];
         ret.v8[2] =  ret.v8[1];
         ret.v8[1] =  ret.v8[0];
         ret.v8[0] =  0;
         expo -= 8;
      }
      if (expo)  ret.v32 <<= expo;
   }
   weGo:
   return ret.v32;
}  // smcGetSectCount()

/** Read the SMC's CID
 *
 *  This function reads the card specific data register (CSD, 16 byte) of an
 *  inserted and powered up small memory card. In case of success 0 is
 *  returned and the result is stored in \ref smcState "smcState.csd".
 *
 *  It is usually not necessary to call this function if \ref smcTypeD() is
 *  true as the OCR and CSD have been read during the card type
 *  determination process.
 *
 *  \param buff 16 byte to write the CID to
 *  \return 0: OK  or negative response of command \ref SEND_CSD or
 *                or 0x80 if SCM isn't powered up.
 */
uint8_t smcReadCID(uint8_t * buff){
   if (!(smcState.cardType & CT_POWERD_UP)) return 0x80;
   uint8_t ret = sendCmd0arg(SEND_CID);
   if (ret) goto weGo;

   uint8_t cnt = 0;
   while (spi2Tranceive(0xFF) != SMC_DAT_TOK) {
     if(++cnt > 24) {
        ret = 0x80;
        goto weGo; // give it up
     }
   }
   spi2ReceiveNS(buff, 16, 2); // 16 bytes of response token, skip CRC
   //   spi2ReceiveSN(NULL, 2, 0);  // skip CRCreceive buffer

#if DEBUG_SMC >= 1
      bufLogTxt_P(smcRdCIDMes);
      bufLogTxt(form16BytSeq(buff), 55);
      bufLogPut2C(' ', '\n');
#endif
   weGo:
   deSelectSMC();
   return ret;
} // smcReadCID()

/*  Determine the card type
 *
 *  This function tries to determine the small memory card's type. The
 *  result is set in \ref smcState "smcState.cardType"
 *  (&amp; \ref CT_TYPE_MASK | CT_HC) and returned.
 *
 *  This function does nothing (except clear
 *  \ref smcState "smcState.cardType") if the card has not been initialised
 *  to \ref smcSetIdle "idle state" once before.
 *
 *  \return 0: no type determined,  \ref CT_V_1,
 *             \ref CT_V_2 [| \ref CT_HC], \ref CT_V_3,
 */
uint8_t getSMCtype(){
   if (!smcInsPow()){
      smcState.flag |= FL_CAT | CT_INS_POWD;
      return 0;
   }
   SAMPLE_MIN_SP(1);
   const uint8_t rT = smcState.cardType & CT_TYPE_MASK;
   if (rT) return rT;
   smcState.flag |= FL_CAT;
   return 0;
} // getSMCtype()

